(*
// Copyright 1999-2021 Intel Corporation All Rights Reserved.
//
// The source code, information and material ("Material") contained herein is
// owned by Intel Corporation or its suppliers or licensors, and title
// to such Material remains with Intel Corporation or its suppliers or
// licensors. The Material contains proprietary information of Intel
// or its suppliers and licensors. The Material is protected by worldwide
// copyright laws and treaty provisions. No part of the Material may be used,
// copied, reproduced, modified, published, uploaded, posted, transmitted,
// distributed or disclosed in any way without Intel's prior express written
// permission. No license under any patent, copyright or other intellectual
// property rights in the Material is granted to or conferred upon you,
// either expressly, by implication, inducement, estoppel or otherwise.
// Any license under such intellectual property rights must be express and
// approved by Intel in writing.
//
// Unless otherwise agreed by Intel in writing,
// you may not remove or alter this notice or any other notice embedded in
// Materials by Intel or Intel's suppliers or licensors in any way.
//
*)

//
//       Intel(R) Integrated Performance Primitives (Intel(R) IPP)
//
// Dependencies:
//       Image Processing (ippi): Core (ippcore), Vector Math (ippvm), Signal Processing (ipps)
//       Computer Vision (ippcv): Image Processing (ippi)
//
// DLLs: 32  64 bit
//       ----------
//       w7, my    Optimized for processors with Intel SSE2
//       s8, n8    Optimized for processors with Supplemental Streaming SIMD Extensions 3 (SSSE3)
//         , m7    Optimized for processors with Intel SSE3
//       p8, y8    Optimized for processors with Intel SSE4.2
//       g9, e9    Optimized for processors with Intel® Advanced Vector Extensions (Intel® AVX) and Intel® Advanced Encryption Standard New Instructions (Intel® AES-NI)
//       h9, l9    Optimized for processors with Intel® Advanced Vector Extensions 2 (Intel® AVX2)
//
// The dispatcher automatically executes the optimal version of each IPP function at run time, thus it is inherent part of dynamic library. Since Intel IPP 9.0, there is no requirement to initialize the dispatcher by calling ippInit() for static linkage anymore.
// If your Intel IPP application uses the standard distribution dynamic library this dispatcher initialization step is handled automatically, as part of the dynamic library initialization.
//
// All Intel® Integrated Performance Primitives functions are thread-safe. They support multi-threading in both dynamic and static libraries and can be used in multi-threaded applications.
// The multi-threaded libraries are deprecated and moving to external threading is recommended.
// User has the option to use different threading models like Intel® TBB, OpenMP and Intel® Cilk Plus etc.
//
// Intel IPP platform-aware functions use _L suffix and are designed to overcome 32-bit size limitations
//  Platform-aware functions cover only the functionality that is implemented in standard Intel IPP functions
//
// Static linking: -Xe (Use external linker in "Additions and Overrides" because internal linker does not support static libraries)
//  {$link ippcoremt.lib}, {$link ippvmmt.lib}, {$link ippsmt.lib}, {$link ippimt.lib}
//  ... external name '_ippInit@0'

unit IPP;

{$IFDEF FPC}
  {$mode objfpc}
  {$macro on}
  {$align 4}
  {$packenum 4}
  {$packrecords C}
{$ENDIF}
{$INLINE ON}

{$DEFINE LOAD_DYNAMICALLY}

interface

{$IFDEF WINDOWS}
 {$DEFINE extdecl:=stdcall}
 const libippcore = 'ippcore.dll';
       libipps = 'ipps.dll';
       libippi = 'ippi.dll';
       libippcv = 'ippcv.dll';
{$ELSE}
 {$DEFINE extdecl:=cdecl}
 const libippcore = 'libippcore.so';
       libipps = 'libipps.so';
       libippi = 'libippi.so';
       libippcv = 'libippcv.so';
{$ENDIF}

(******************************************************************************
 *               Basic Types and Macro Definitions (ippbase.h)                *
 ******************************************************************************)
type
  int = integer;
  Pint = ^int;
  float = single;

  Ipp8u = byte;
  Ipp16u = word;
  Ipp32u = dword;
  Ipp8s = shortint;
  Ipp16s = smallint;
  Ipp32s = integer;
  Ipp32f = single;
  Ipp64s = Int64;
  Ipp64u = UInt64;
  Ipp64f = double;
  Ipp16f = Ipp16s;

  PIpp8u = ^Ipp8u;
  PIpp16u = ^Ipp16u;
  PIpp32u = ^Ipp32u;
  PIpp16s = ^Ipp16s;
  PIpp32s = ^Ipp32s;
  PIpp32f = ^Ipp32f;
  PIpp64f = ^Ipp64f;

  array_double_2_2 = array[0..1, 0..1] of double;
  array_double_2_3 = array[0..1, 0..2] of double;
  array_double_4_2 = array[0..3, 0..1] of double;

  IppLibraryVersion = record
      major: integer;                   { e.g. 1                               }
      minor: integer;                   { e.g. 2                               }
      majorBuild: integer;              { e.g. 3                               }
      build: integer;                   { e.g. 10, always >= majorBuild        }
      targetCpu: array [0..4-1] of AnsiChar; { corresponding to Intel(R) processor  }
      Name: PAnsiChar;                  { e.g. "ippsw7"                        }
      Version: PAnsiChar;               { e.g. "v1.2 Beta"                     }
      BuildDate: PAnsiChar;             { e.g. "Jul 20 99"                     }
  end;
  PIppLibraryVersion = ^IppLibraryVersion;

  Ipp32fc = record
    re,
    im: Ipp32f;
  end;
  PIpp32fc = ^Ipp32fc;

  IppDataType = (
     ippDTUndef = -1,
     ippDT1u    =  0,
     ippDT8u    =  1,
     ippDT8uc   =  2,
     ippDT8s    =  3,
     ippDT8sc   =  4,
     ippDT16u   =  5,
     ippDT16uc  =  6,
     ippDT16s   =  7,
     ippDT16sc  =  8,
     ippDT32u   =  9,
     ippDT32uc  = 10,
     ippDT32s   = 11,
     ippDT32sc  = 12,
     ippDT32f   = 13,
     ippDT32fc  = 14,
     ippDT64u   = 15,
     ippDT64uc  = 16,
     ippDT64s   = 17,
     ippDT64sc  = 18,
     ippDT64f   = 19,
     ippDT64fc  = 20
  );

  IppBool = (ippFalse = 0, ippTrue = 1);

(******************************************************************************
 *             Derivative Types and Macro Definitions (ipptypes.h)            *
 ******************************************************************************)

const
  ippCPUID_MMX      = $00000001;   (* Intel Architecture MMX technology supported  *)
  ippCPUID_SSE      = $00000002;   (* Streaming SIMD Extensions                    *)
  ippCPUID_SSE2     = $00000004;   (* Streaming SIMD Extensions 2                  *)
  ippCPUID_SSE3     = $00000008;   (* Streaming SIMD Extensions 3                  *)
  ippCPUID_SSSE3    = $00000010;   (* Supplemental Streaming SIMD Extensions 3     *)
  ippCPUID_MOVBE    = $00000020;   (* The processor supports MOVBE instruction     *)
  ippCPUID_SSE41    = $00000040;   (* Streaming SIMD Extensions 4.1                *)
  ippCPUID_SSE42    = $00000080;   (* Streaming SIMD Extensions 4.2                *)
  ippCPUID_AVX      = $00000100;   (* Advanced Vector Extensions instruction set   *)
  ippAVX_ENABLEDBYOS= $00000200;   (* The operating system supports AVX            *)
  ippCPUID_AES      = $00000400;   (* AES instruction                              *)
  ippCPUID_CLMUL    = $00000800;   (* PCLMULQDQ instruction                        *)
  ippCPUID_ABR      = $00001000;   (* Reserved                                     *)
  ippCPUID_RDRAND   = $00002000;   (* Read Random Number instructions              *)
  ippCPUID_F16C     = $00004000;   (* Float16 instructions                         *)
  ippCPUID_AVX2     = $00008000;   (* Advanced Vector Extensions 2 instruction set *)

  IPP_FFT_DIV_FWD_BY_N = 1;
  IPP_FFT_DIV_INV_BY_N = 2;
  IPP_FFT_DIV_BY_SQRTN = 4;
  IPP_FFT_NODIV_BY_ANY = 8;

  IPP_DIV_FWD_BY_N = 1;
  IPP_DIV_INV_BY_N = 2;
  IPP_DIV_BY_SQRTN = 4;
  IPP_NODIV_BY_ANY = 8;

  IPP_FASTN_ORIENTATION = $0001;
  IPP_FASTN_NMS         = $0002;
  IPP_FASTN_CIRCLE      = $0004;
  IPP_FASTN_SCORE_MODE0 = $0020;

  IPP_SEGMENT_QUEUE     = $01;
  IPP_SEGMENT_DISTANCE  = $02;
  IPP_SEGMENT_BORDER_4  = $40;
  IPP_SEGMENT_BORDER_8  = $80;

type

  IppRoundMode = (
    ippRndZero,
    ippRndNear,
    ippRndFinancial,
    ippRndHintAccurate=$10
  );

  IppHintAlgorithm = (
    ippAlgHintNone,
    ippAlgHintFast,
    ippAlgHintAccurate
  );

  IppCmpOp = (
    ippCmpLess,
    ippCmpLessEq,
    ippCmpEq,
    ippCmpGreaterEq,
    ippCmpGreater
  );

  IppNormType = (
    ippNormInf = $00000001,
    ippNormL1  = $00000002,
    ippNormL2  = $00000004
  );

  IppPointPolar = record
    rho: Ipp32f;
    theta: Ipp32f;
  end;
  PIppPointPolar = ^IppPointPolar;

  IppiBorderType = (
    ippBorderRepl         =  1,
    ippBorderWrap         =  2,
    ippBorderMirror       =  3, (* left border: 012... -> 21012... *)
    ippBorderMirrorR      =  4, (* left border: 012... -> 210012... *)
    ippBorderDefault      =  5,
    ippBorderConst        =  6,
    ippBorderTransp       =  7,

    (* Flags to use source image memory pixels from outside of the border in particular directions *)
    ippBorderInMemTop     =  $0010,
    ippBorderInMemBottom  =  $0020,
    ippBorderInMemLeft    =  $0040,
    ippBorderInMemRight   =  $0080,
    ippBorderInMem        =  ippBorderInMemLeft + ippBorderInMemTop + ippBorderInMemRight + ippBorderInMemBottom,

    (* Flags to use source image memory pixels from outside of the border for first stage only in multi-stage filters *)
    ippBorderFirstStageInMem = $0F00
  );

  IppiRect = record
    x: int;
    y: int;
    width: int;
    height: int;
  end;

  IppiPoint = record
    x: int;
    y: int;
  end;
  PIppiPoint = ^IppiPoint;

  IppiSize = record
    width: int;
    height: int;
  end;

  IppiMaskSize = (
      ippMskSize1x3 = 13,
      ippMskSize1x5 = 15,
      ippMskSize3x1 = 31,
      ippMskSize3x3 = 33,
      ippMskSize5x1 = 51,
      ippMskSize5x5 = 55
  );


(*
      IPPI_INTER_NN     = 1,
      IPPI_INTER_LINEAR = 2,
      IPPI_INTER_CUBIC  = 4,
      IPPI_INTER_CUBIC2P_BSPLINE,     /* two-parameter cubic filter (B=1, C=0) */
      IPPI_INTER_CUBIC2P_CATMULLROM,  /* two-parameter cubic filter (B=0, C=1/2) */
      IPPI_INTER_CUBIC2P_B05C03,      /* two-parameter cubic filter (B=1/2, C=3/10) */
      IPPI_INTER_SUPER  = 8,
      IPPI_INTER_LANCZOS = 16,
      IPPI_ANTIALIASING  = (1 << 29),
      IPPI_SUBPIXEL_EDGE = (1 << 30),
      IPPI_SMOOTH_EDGE   = (1 << 31)
*)

  IppiInterpolationType = (
      ippHahn = 0,
      ippNearest = 1,
      ippLinear = 2,
      ippCubic = 6,
      ippSuper = 8,
      ippLanczos = 16
  );

  TIppiDeconvLR_32f_C1R = record end;
  PIppiDeconvLR_32f_C1R = ^TIppiDeconvLR_32f_C1R;

  IppiWarpDirection = (
      ippWarpForward,
      ippWarpBackward
  );

  IppiResizeSpec_32f = record end;
  PIppiResizeSpec_32f = ^IppiResizeSpec_32f;

  IppiWarpSpec = record end;
  PIppiWarpSpec = ^IppiWarpSpec;

  IppiFilterBorderSpec = record end;
  PIppiFilterBorderSpec = ^IppiFilterBorderSpec;

  IppiHistogramSpec = record end;
  PIppiHistogramSpec = ^IppiHistogramSpec;

  IppiHOGConfig = record
     cvCompatible: int;   (* openCV compatible output format *)
     cellSize: int;       (* squre cell size (pixels) *)
     blockSize: int;      (* square block size (pixels) *)
     blockStride: int;    (* block displacement (the same for x- and y- directions) *)
     nbins: int;          (* required number of bins *)
     sigma: Ipp32f;       (* gaussian factor of HOG block weights *)
     l2thresh: Ipp32f;    (* normalization factor *)
     winSize: IppiSize;   (* detection window size (pixels) *)
  end;
  PIppiHOGConfig = ^IppiHOGConfig;

  IppiHOGSpec = record end;
  PIppiHOGSpec = ^IppiHOGSpec;

  IppiFFTSpec_R_32f = record end;
  PIppiFFTSpec_R_32f = ^IppiFFTSpec_R_32f;

  IppiDFTSpec_R_32f = record end;
  PIppiDFTSpec_R_32f = ^IppiDFTSpec_R_32f;

  IppiDFTSpec_C_32fc = record end;
  PIppiDFTSpec_C_32fc = ^IppiDFTSpec_C_32fc;

  IppiMomentState_64f = record end;
  PIppiMomentState_64f = ^IppiMomentState_64f;
  PIppiHuMoment_64f = ^Ipp64f;

  IppFilterGaussianSpec = record end;
  PIppFilterGaussianSpec = ^IppFilterGaussianSpec;

  IppiMorphState = record end;
  PIppiMorphState = ^IppiMorphState;

  IppiMorphAdvState = record end;
  PIppiMorphAdvState = ^IppiMorphAdvState;

  IppiDifferentialKernel = (
    ippFilterSobelVert,
    ippFilterSobelHoriz,
    ippFilterSobel,
    ippFilterScharrVert,
    ippFilterScharrHoriz,
    ippFilterScharr,
    ippFilterCentralDiffVert,
    ippFilterCentralDiffHoriz,
    ippFilterCentralDiff
  );

  IppiNorm = (
    ippiNormInf = 0,
    ippiNormL1 = 1,
    ippiNormL2 = 2,
    ippiNormFM = 3
  );

  IppiConnectedComp = record
    area: Ipp64f;                 (*  area of the segmented component  *)
    value: array[0..2] of Ipp64f; (*  gray scale value of the segmented component  *)
    rect: IppiRect;               (*  bounding rectangle of the segmented component  *)
  end;
  PIppiConnectedComp = ^IppiConnectedComp;

  IppiFastNSpec = record end;
  PIppiFastNSpec = ^IppiFastNSpec;

  (* /////////////////////////////////////////////////////////////////////////////
  //        The following enumerator defines a status of Intel(R) IPP operations
  //                     negative value means error
  *)
  IppStatus = (
    (* errors *)
    ippStsNotSupportedModeErr    = -9999,(* The requested mode is currently not supported.  *)
    ippStsCpuNotSupportedErr     = -9998,(* The target CPU is not supported. *)
    ippStsInplaceModeNotSupportedErr = -9997,(* The inplace operation is currently not supported. *)

    ippStsABIErrXMM15            = -8017, (* XMM15 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM14            = -8016, (* XMM14 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM13            = -8015, (* XMM13 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM12            = -8014, (* XMM12 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM11            = -8013, (* XMM11 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM10            = -8012, (* XMM10 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM9             = -8011, (* XMM9 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM8             = -8010, (* XMM8 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM7             = -8009, (* XMM7 is not saved by Intel(R) IPP function *)
    ippStsABIErrXMM6             = -8008, (* XMM6 is not saved by Intel(R) IPP function *)
    ippStsABIErrR15              = -8007, (* R15 is not saved by Intel(R) IPP function *)
    ippStsABIErrR14              = -8006, (* R14 is not saved by Intel(R) IPP function *)
    ippStsABIErrR13              = -8005, (* R13 is not saved by Intel(R) IPP function *)
    ippStsABIErrR12              = -8004, (* R12 is not saved by Intel(R) IPP function *)
    ippStsABIErrRBP              = -8003, (* RBP is not saved by Intel(R) IPP function *)
    ippStsABIErrRSI              = -8002, (* RSI is not saved by Intel(R) IPP function *)
    ippStsABIErrRDI              = -8001, (* RDI is not saved by Intel(R) IPP function *)
    ippStsABIErrRBX              = -8000, (* RBX is not saved by Intel(R) IPP function *)

    ippStsIIRIIRLengthErr        = -234, (* Vector length for IIRIIR function is less than 3*(IIR order) *)
    ippStsWarpTransformTypeErr   = -233, (* The warp transform type is illegal *)
    ippStsExceededSizeErr        = -232, (* Requested size exceeded the maximum supported ROI size *)
    ippStsWarpDirectionErr       = -231, (* The warp transform direction is illegal *)

    ippStsFilterTypeErr          = -230, (* The filter type is incorrect or not supported *)

    ippStsNormErr                = -229, (* The norm is incorrect or not supported *)

    ippStsAlgTypeErr             = -228, (* Algorithm type is not supported.        *)
    ippStsMisalignedOffsetErr    = -227, (* The offset is not aligned with an element. *)

    ippStsQuadraticNonResidueErr = -226, (* SQRT operation on quadratic non-residue value. *)

    ippStsBorderErr              = -225, (* Illegal value for border type.*)

    ippStsDitherTypeErr          = -224, (* Dithering type is not supported. *)
    ippStsH264BufferFullErr      = -223, (* Buffer for the output bitstream is full. *)
    ippStsWrongAffinitySettingErr= -222, (* An affinity setting does not correspond to the affinity setting that was set by f.ippSetAffinity(). *)
    ippStsLoadDynErr             = -221, (* Error when loading the dynamic library. *)

    ippStsPointAtInfinity        = -220, (* Point at infinity is detected.  *)

    ippStsUnknownStatusCodeErr   = -216, (* Unknown status code. *)

    ippStsOFBSizeErr             = -215, (* Incorrect value for crypto OFB block size. *)
    ippStsLzoBrokenStreamErr     = -214, (* LZO safe decompression function cannot decode LZO stream. *)

    ippStsRoundModeNotSupportedErr  = -213, (* Rounding mode is not supported. *)
    ippStsDecimateFractionErr    = -212, (* Fraction in Decimate is not supported. *)
    ippStsWeightErr              = -211, (* Incorrect value for weight. *)

    ippStsQualityIndexErr        = -210, (* Cannot calculate the quality index for an image filled with a constant. *)
    ippStsIIRPassbandRippleErr   = -209, (* Ripple in passband for Chebyshev1 design is less than zero, equal to zero, or greater than 29. *)
    ippStsFilterFrequencyErr     = -208, (* Cutoff frequency of filter is less than zero, equal to zero, or greater than 0.5. *)
    ippStsFIRGenOrderErr         = -207, (* Order of the FIR filter for design is less than 1.                    *)
    ippStsIIRGenOrderErr         = -206, (* Order of the IIR filter for design is less than 1, or greater than 12. *)

    ippStsConvergeErr            = -205, (* The algorithm does not converge. *)
    ippStsSizeMatchMatrixErr     = -204, (* The sizes of the source matrices are unsuitable. *)
    ippStsCountMatrixErr         = -203, (* Count value is less than, or equal to zero. *)
    ippStsRoiShiftMatrixErr      = -202, (* RoiShift value is negative or not divisible by the size of the data type. *)

    ippStsResizeNoOperationErr   = -201, (* One of the output image dimensions is less than 1 pixel. *)
    ippStsSrcDataErr             = -200, (* The source buffer contains unsupported data. *)
    ippStsMaxLenHuffCodeErr      = -199, (* Huff: Max length of Huffman code is more than the expected one. *)
    ippStsCodeLenTableErr        = -198, (* Huff: Invalid codeLenTable. *)
    ippStsFreqTableErr           = -197, (* Huff: Invalid freqTable. *)

    ippStsIncompleteContextErr   = -196, (* Crypto: set up of context is not complete. *)

    ippStsSingularErr            = -195, (* Matrix is singular. *)
    ippStsSparseErr              = -194, (* Positions of taps are not in ascending order, or are negative, or repetitive. *)
    ippStsBitOffsetErr           = -193, (* Incorrect bit offset value. *)
    ippStsQPErr                  = -192, (* Incorrect quantization parameter value. *)
    ippStsVLCErr                 = -191, (* Illegal VLC or FLC is detected during stream decoding. *)
    ippStsRegExpOptionsErr       = -190, (* RegExp: Options for the pattern are incorrect. *)
    ippStsRegExpErr              = -189, (* RegExp: The structure pRegExpState contains incorrect data. *)
    ippStsRegExpMatchLimitErr    = -188, (* RegExp: The match limit is exhausted. *)
    ippStsRegExpQuantifierErr    = -187, (* RegExp: Incorrect quantifier. *)
    ippStsRegExpGroupingErr      = -186, (* RegExp: Incorrect grouping. *)
    ippStsRegExpBackRefErr       = -185, (* RegExp: Incorrect back reference. *)
    ippStsRegExpChClassErr       = -184, (* RegExp: Incorrect character class. *)
    ippStsRegExpMetaChErr        = -183, (* RegExp: Incorrect metacharacter. *)
    ippStsStrideMatrixErr        = -182,  (* Stride value is not positive or not divisible by the size of the data type. *)
    ippStsCTRSizeErr             = -181,  (* Incorrect value for crypto CTR block size. *)
    ippStsJPEG2KCodeBlockIsNotAttached =-180, (* Codeblock parameters are not attached to the state structure. *)
    ippStsNotPosDefErr           = -179,      (* Matrix is not positive definite. *)

    ippStsEphemeralKeyErr        = -178, (* ECC: Invalid ephemeral key.   *)
    ippStsMessageErr             = -177, (* ECC: Invalid message digest.  *)
    ippStsShareKeyErr            = -176, (* ECC: Invalid share key.   *)
    ippStsIvalidPublicKey        = -175, (* ECC: Invalid public key.  *)
    ippStsIvalidPrivateKey       = -174, (* ECC: Invalid private key. *)
    ippStsOutOfECErr             = -173, (* ECC: Point out of EC.     *)
    ippStsECCInvalidFlagErr      = -172, (* ECC: Invalid Flag.        *)

    ippStsMP3FrameHeaderErr      = -171,  (* Error in fields of the IppMP3FrameHeader structure. *)
    ippStsMP3SideInfoErr         = -170,  (* Error in fields of the IppMP3SideInfo structure. *)

    ippStsBlockStepErr           = -169,  (* Step for Block is less than 8. *)
    ippStsMBStepErr              = -168,  (* Step for MB is less than 16. *)

    ippStsAacPrgNumErr           = -167,  (* AAC: Invalid number of elements for one program.   *)
    ippStsAacSectCbErr           = -166,  (* AAC: Invalid section codebook.                     *)
    ippStsAacSfValErr            = -164,  (* AAC: Invalid scalefactor value.                    *)
    ippStsAacCoefValErr          = -163,  (* AAC: Invalid quantized coefficient value.          *)
    ippStsAacMaxSfbErr           = -162,  (* AAC: Invalid coefficient index.  *)
    ippStsAacPredSfbErr          = -161,  (* AAC: Invalid predicted coefficient index.  *)
    ippStsAacPlsDataErr          = -160,  (* AAC: Invalid pulse data attributes.  *)
    ippStsAacGainCtrErr          = -159,  (* AAC: Gain control is not supported.  *)
    ippStsAacSectErr             = -158,  (* AAC: Invalid number of sections.  *)
    ippStsAacTnsNumFiltErr       = -157,  (* AAC: Invalid number of TNS filters.  *)
    ippStsAacTnsLenErr           = -156,  (* AAC: Invalid length of TNS region.  *)
    ippStsAacTnsOrderErr         = -155,  (* AAC: Invalid order of TNS filter.  *)
    ippStsAacTnsCoefResErr       = -154,  (* AAC: Invalid bit-resolution for TNS filter coefficients.  *)
    ippStsAacTnsCoefErr          = -153,  (* AAC: Invalid coefficients of TNS filter. *)
    ippStsAacTnsDirectErr        = -152,  (* AAC: Invalid direction TNS filter.  *)
    ippStsAacTnsProfileErr       = -151,  (* AAC: Invalid TNS profile.  *)
    ippStsAacErr                 = -150,  (* AAC: Internal error.  *)
    ippStsAacBitOffsetErr        = -149,  (* AAC: Invalid current bit offset in bitstream.  *)
    ippStsAacAdtsSyncWordErr     = -148,  (* AAC: Invalid ADTS syncword.  *)
    ippStsAacSmplRateIdxErr      = -147,  (* AAC: Invalid sample rate index.  *)
    ippStsAacWinLenErr           = -146,  (* AAC: Invalid window length (not short or long).  *)
    ippStsAacWinGrpErr           = -145,  (* AAC: Invalid number of groups for current window length.  *)
    ippStsAacWinSeqErr           = -144,  (* AAC: Invalid window sequence range.  *)
    ippStsAacComWinErr           = -143,  (* AAC: Invalid common window flag.  *)
    ippStsAacStereoMaskErr       = -142,  (* AAC: Invalid stereo mask.  *)
    ippStsAacChanErr             = -141,  (* AAC: Invalid channel number.  *)
    ippStsAacMonoStereoErr       = -140,  (* AAC: Invalid mono-stereo flag.  *)
    ippStsAacStereoLayerErr      = -139,  (* AAC: Invalid this Stereo Layer flag.  *)
    ippStsAacMonoLayerErr        = -138,  (* AAC: Invalid this Mono Layer flag.  *)
    ippStsAacScalableErr         = -137,  (* AAC: Invalid scalable object flag.  *)
    ippStsAacObjTypeErr          = -136,  (* AAC: Invalid audio object type.  *)
    ippStsAacWinShapeErr         = -135,  (* AAC: Invalid window shape.  *)
    ippStsAacPcmModeErr          = -134,  (* AAC: Invalid PCM output interleaving indicator.  *)
    ippStsVLCUsrTblHeaderErr          = -133,  (* VLC: Invalid header inside table. *)
    ippStsVLCUsrTblUnsupportedFmtErr  = -132,  (* VLC: Table format is not supported.  *)
    ippStsVLCUsrTblEscAlgTypeErr      = -131,  (* VLC: Ecs-algorithm is not supported. *)
    ippStsVLCUsrTblEscCodeLengthErr   = -130,  (* VLC: Esc-code length inside table header is incorrect. *)
    ippStsVLCUsrTblCodeLengthErr      = -129,  (* VLC: Code length inside table is incorrect.  *)
    ippStsVLCInternalTblErr           = -128,  (* VLC: Invalid internal table. *)
    ippStsVLCInputDataErr             = -127,  (* VLC: Invalid input data. *)
    ippStsVLCAACEscCodeLengthErr      = -126,  (* VLC: Invalid AAC-Esc code length. *)
    ippStsNoiseRangeErr         = -125,  (* Noise value for Wiener Filter is out of range. *)
    ippStsUnderRunErr           = -124,  (* Error in data under run. *)
    ippStsPaddingErr            = -123,  (* Detected padding error indicates the possible data corruption. *)
    ippStsCFBSizeErr            = -122,  (* Incorrect value for crypto CFB block size. *)
    ippStsPaddingSchemeErr      = -121,  (* Invalid padding scheme.  *)
    ippStsInvalidCryptoKeyErr   = -120,  (* A compromised key causes suspansion of the requested cryptographic operation.  *)
    ippStsLengthErr             = -119,  (* Incorrect value for string length. *)
    ippStsBadModulusErr         = -118,  (* Bad modulus caused a failure in module inversion. *)
    ippStsLPCCalcErr            = -117,  (* Cannot evaluate linear prediction. *)
    ippStsRCCalcErr             = -116,  (* Cannot compute reflection coefficients. *)
    ippStsIncorrectLSPErr       = -115,  (* Incorrect values for Linear Spectral Pair. *)
    ippStsNoRootFoundErr        = -114,  (* No roots are found for equation. *)
    ippStsJPEG2KBadPassNumber   = -113,  (* Pass number exceeds allowed boundaries [0,nOfPasses-1]. *)
    ippStsJPEG2KDamagedCodeBlock= -112,  (* Codeblock for decoding contains damaged data. *)
    ippStsH263CBPYCodeErr       = -111,  (* Illegal Huffman code is detected through CBPY stream processing. *)
    ippStsH263MCBPCInterCodeErr = -110,  (* Illegal Huffman code is detected through MCBPC Inter stream processing. *)
    ippStsH263MCBPCIntraCodeErr = -109,  (* Illegal Huffman code is detected through MCBPC Intra stream processing. *)
    ippStsNotEvenStepErr        = -108,  (* Step value is not pixel multiple. *)
    ippStsHistoNofLevelsErr     = -107,  (* Number of levels for histogram is less than 2. *)
    ippStsLUTNofLevelsErr       = -106,  (* Number of levels for LUT is less than 2. *)
    ippStsMP4BitOffsetErr       = -105,  (* Incorrect bit offset value. *)
    ippStsMP4QPErr              = -104,  (* Incorrect quantization parameter. *)
    ippStsMP4BlockIdxErr        = -103,  (* Incorrect block index. *)
    ippStsMP4BlockTypeErr       = -102,  (* Incorrect block type. *)
    ippStsMP4MVCodeErr          = -101,  (* Illegal Huffman code is detected during MV stream processing. *)
    ippStsMP4VLCCodeErr         = -100,  (* Illegal Huffman code is detected during VLC stream processing. *)
    ippStsMP4DCCodeErr          = -99,   (* Illegal code is detected during DC stream processing. *)
    ippStsMP4FcodeErr           = -98,   (* Incorrect fcode value. *)
    ippStsMP4AlignErr           = -97,   (* Incorrect buffer alignment .           *)
    ippStsMP4TempDiffErr        = -96,   (* Incorrect temporal difference.         *)
    ippStsMP4BlockSizeErr       = -95,   (* Incorrect size of a block or macroblock. *)
    ippStsMP4ZeroBABErr         = -94,   (* All BAB values are equal to zero.             *)
    ippStsMP4PredDirErr         = -93,   (* Incorrect prediction direction.        *)
    ippStsMP4BitsPerPixelErr    = -92,   (* Incorrect number of bits per pixel.    *)
    ippStsMP4VideoCompModeErr   = -91,   (* Incorrect video component mode.       *)
    ippStsMP4LinearModeErr      = -90,   (* Incorrect DC linear mode. *)
    ippStsH263PredModeErr       = -83,   (* Incorrect Prediction Mode value.                                       *)
    ippStsH263BlockStepErr      = -82,   (* The step value is less than 8.                                         *)
    ippStsH263MBStepErr         = -81,   (* The step value is less than 16.                                        *)
    ippStsH263FrameWidthErr     = -80,   (* The frame width is less than 8.                                        *)
    ippStsH263FrameHeightErr    = -79,   (* The frame height is less than, or equal to zero.                        *)
    ippStsH263ExpandPelsErr     = -78,   (* Expand pixels number is less than 8.                               *)
    ippStsH263PlaneStepErr      = -77,   (* Step value is less than the plane width.                           *)
    ippStsH263QuantErr          = -76,   (* Quantizer value is less than, or equal to zero, or greater than 31. *)
    ippStsH263MVCodeErr         = -75,   (* Illegal Huffman code is detected during MV stream processing.                  *)
    ippStsH263VLCCodeErr        = -74,   (* Illegal Huffman code is detected during VLC stream processing.                 *)
    ippStsH263DCCodeErr         = -73,   (* Illegal code is detected during DC stream processing.                          *)
    ippStsH263ZigzagLenErr      = -72,   (* Zigzag compact length is more than 64.                             *)
    ippStsFBankFreqErr          = -71,   (* Incorrect value for the filter bank frequency parameter. *)
    ippStsFBankFlagErr          = -70,   (* Incorrect value for the filter bank parameter.           *)
    ippStsFBankErr              = -69,   (* Filter bank is not correctly initialized.              *)
    ippStsNegOccErr             = -67,   (* Occupation count is negative.                     *)
    ippStsCdbkFlagErr           = -66,   (* Incorrect value for the codebook flag parameter. *)
    ippStsSVDCnvgErr            = -65,   (* SVD algorithm does not converge.               *)
    ippStsJPEGHuffTableErr      = -64,   (* JPEG Huffman table is destroyed.        *)
    ippStsJPEGDCTRangeErr       = -63,   (* JPEG DCT coefficient is out of range. *)
    ippStsJPEGOutOfBufErr       = -62,   (* Attempt to access out of the buffer limits.   *)
    ippStsDrawTextErr           = -61,   (* System error in the draw text operation. *)
    ippStsChannelOrderErr       = -60,   (* Incorrect order of the destination channels. *)
    ippStsZeroMaskValuesErr     = -59,   (* All values of the mask are equal to zero. *)
    ippStsQuadErr               = -58,   (* The quadrangle is nonconvex or degenerates into triangle, line, or point *)
    ippStsRectErr               = -57,   (* Size of the rectangle region is less than, or equal to 1. *)
    ippStsCoeffErr              = -56,   (* Incorrect values for transformation coefficients.   *)
    ippStsNoiseValErr           = -55,   (* Incorrect value for noise amplitude for dithering.             *)
    ippStsDitherLevelsErr       = -54,   (* Number of dithering levels is out of range.             *)
    ippStsNumChannelsErr        = -53,   (* Number of channels is incorrect, or not supported.                  *)
    ippStsCOIErr                = -52,   (* COI is out of range. *)
    ippStsDivisorErr            = -51,   (* Divisor is equal to zero, function is aborted. *)
    ippStsAlphaTypeErr          = -50,   (* Illegal type of image compositing operation.                           *)
    ippStsGammaRangeErr         = -49,   (* Gamma range bounds is less than, or equal to zero.                      *)
    ippStsGrayCoefSumErr        = -48,   (* Sum of the conversion coefficients must be less than, or equal to 1.    *)
    ippStsChannelErr            = -47,   (* Illegal channel number.                                                *)
    ippStsToneMagnErr           = -46,   (* Tone magnitude is less than, or equal to zero.                          *)
    ippStsToneFreqErr           = -45,   (* Tone frequency is negative, or greater than, or equal to 0.5.           *)
    ippStsTonePhaseErr          = -44,   (* Tone phase is negative, or greater than, or equal to 2*PI.              *)
    ippStsTrnglMagnErr          = -43,   (* Triangle magnitude is less than, or equal to zero.                      *)
    ippStsTrnglFreqErr          = -42,   (* Triangle frequency is negative, or greater than, or equal to 0.5.       *)
    ippStsTrnglPhaseErr         = -41,   (* Triangle phase is negative, or greater than, or equal to 2*PI.          *)
    ippStsTrnglAsymErr          = -40,   (* Triangle asymmetry is less than -PI, or greater than, or equal to PI.   *)
    ippStsHugeWinErr            = -39,   (* Kaiser window is too big.                                             *)
    ippStsJaehneErr             = -38,   (* Magnitude value is negative.                                           *)
    ippStsStrideErr             = -37,   (* Stride value is less than the length of the row. *)
    ippStsEpsValErr             = -36,   (* Negative epsilon value.             *)
    ippStsWtOffsetErr           = -35,   (* Invalid offset value for wavelet filter.                                       *)
    ippStsAnchorErr             = -34,   (* Anchor point is outside the mask.                                             *)
    ippStsMaskSizeErr           = -33,   (* Invalid mask size.                                                           *)
    ippStsShiftErr              = -32,   (* Shift value is less than zero.                                                *)
    ippStsSampleFactorErr       = -31,   (* Sampling factor is less than, or equal to zero.                                *)
    ippStsSamplePhaseErr        = -30,   (* Phase value is out of range: 0 <= phase < factor.                             *)
    ippStsFIRMRFactorErr        = -29,   (* MR FIR sampling factor is less than, or equal to zero.                         *)
    ippStsFIRMRPhaseErr         = -28,   (* MR FIR sampling phase is negative, or greater than, or equal to the sampling factor. *)
    ippStsRelFreqErr            = -27,   (* Relative frequency value is out of range.                                     *)
    ippStsFIRLenErr             = -26,   (* Length of a FIR filter is less than, or equal to zero.                         *)
    ippStsIIROrderErr           = -25,   (* Order of an IIR filter is not valid. *)
    ippStsDlyLineIndexErr       = -24,   (* Invalid value for the delay line sample index. *)
    ippStsResizeFactorErr       = -23,   (* Resize factor(s) is less than, or equal to zero. *)
    ippStsInterpolationErr      = -22,   (* Invalid interpolation mode. *)
    ippStsMirrorFlipErr         = -21,   (* Invalid flip mode.                                         *)
    ippStsMoment00ZeroErr       = -20,   (* Moment value M(0,0) is too small to continue calculations. *)
    ippStsThreshNegLevelErr     = -19,   (* Negative value of the level in the threshold operation.    *)
    ippStsThresholdErr          = -18,   (* Invalid threshold bounds. *)
    ippStsContextMatchErr       = -17,   (* Context parameter does not match the operation. *)
    ippStsFftFlagErr            = -16,   (* Invalid value for the FFT flag parameter. *)
    ippStsFftOrderErr           = -15,   (* Invalid value for the FFT order parameter. *)
    ippStsStepErr               = -14,   (* Step value is not valid. *)
    ippStsScaleRangeErr         = -13,   (* Scale bounds are out of range. *)
    ippStsDataTypeErr           = -12,   (* Data type is incorrect or not supported. *)
    ippStsOutOfRangeErr         = -11,   (* Argument is out of range, or point is outside the image. *)
    ippStsDivByZeroErr          = -10,   (* An attempt to divide by zero. *)
    ippStsMemAllocErr           = -9,    (* Memory allocated for the operation is not enough.*)
    ippStsNullPtrErr            = -8,    (* Null pointer error. *)
    ippStsRangeErr              = -7,    (* Incorrect values for bounds: the lower bound is greater than the upper bound. *)
    ippStsSizeErr               = -6,    (* Incorrect value for data size. *)
    ippStsBadArgErr             = -5,    (* Incorrect arg/param of the function.  *)
    ippStsNoMemErr              = -4,    (* Not enough memory for the operation. *)
    ippStsSAReservedErr3        = -3,    (* Unknown/unspecified error, -3. *)
    ippStsErr                   = -2,    (* Unknown/unspecified error, -2. *)
    ippStsSAReservedErr1        = -1,    (* Unknown/unspecified error, -1. *)

     (* no errors *)
    ippStsNoErr                 =   0,   (* No errors. *)

     (* warnings  *)
    ippStsNoOperation       =   1,       (* No operation has been executed. *)
    ippStsMisalignedBuf     =   2,       (* Misaligned pointer in operation in which it must be aligned. *)
    ippStsSqrtNegArg        =   3,       (* Negative value(s) for the argument in the Sqrt function. *)
    ippStsInvZero           =   4,       (* INF result. Zero value was met by InvThresh with zero level. *)
    ippStsEvenMedianMaskSize=   5,       (* Even size of the Median Filter mask was replaced with the odd one. *)
    ippStsDivByZero         =   6,       (* Zero value(s) for the divisor in the Div function. *)
    ippStsLnZeroArg         =   7,       (* Zero value(s) for the argument in the Ln function.     *)
    ippStsLnNegArg          =   8,       (* Negative value(s) for the argument in the Ln function. *)
    ippStsNanArg            =   9,       (* Argument value is not a number.                  *)
    ippStsJPEGMarker        =   10,      (* JPEG marker in the bitstream.                 *)
    ippStsResFloor          =   11,      (* All result values are floored.                        *)
    ippStsOverflow          =   12,      (* Overflow in the operation.                   *)
    ippStsLSFLow            =   13,      (* Quantized LP synthesis filter stability check is applied at the low boundary of [0,pi]. *)
    ippStsLSFHigh           =   14,      (* Quantized LP synthesis filter stability check is applied at the high boundary of [0,pi]. *)
    ippStsLSFLowAndHigh     =   15,      (* Quantized LP synthesis filter stability check is applied at both boundaries of [0,pi]. *)
    ippStsZeroOcc           =   16,      (* Zero occupation count. *)
    ippStsUnderflow         =   17,      (* Underflow in the operation. *)
    ippStsSingularity       =   18,      (* Singularity in the operation.                                       *)
    ippStsDomain            =   19,      (* Argument is out of the function domain.                                      *)
    ippStsNonIntelCpu       =   20,      (* The target CPU is not Genuine Intel.                                         *)
    ippStsCpuMismatch       =   21,      (* Cannot set the library for the given CPU.                                     *)
    ippStsNoIppFunctionFound =  22,      (* Application does not contain Intel(R) IPP function calls.                            *)
    ippStsDllNotFoundBestUsed = 23,      (* Dispatcher cannot find the newest version of the Intel(R) IPP dll.                  *)
    ippStsNoOperationInDll  =   24,      (* The function does nothing in the dynamic version of the library.             *)
    ippStsInsufficientEntropy=  25,      (* Generation of the prime/key failed due to insufficient entropy in the random seed and stimulus bit string. *)
    ippStsOvermuchStrings   =   26,      (* Number of destination strings is more than expected.                         *)
    ippStsOverlongString    =   27,      (* Length of one of the destination strings is more than expected.              *)
    ippStsAffineQuadChanged =   28,      (* 4th vertex of destination quad is not equal to customer's one.               *)
    ippStsWrongIntersectROI =   29,      (* ROI has no intersection with the source or destination ROI. No operation. *)
    ippStsWrongIntersectQuad =  30,      (* Quadrangle has no intersection with the source or destination ROI. No operation. *)
    ippStsSmallerCodebook   =   31,      (* Size of created codebook is less than the cdbkSize argument. *)
    ippStsSrcSizeLessExpected = 32,      (* DC: Size of the source buffer is less than the expected one. *)
    ippStsDstSizeLessExpected = 33,      (* DC: Size of the destination buffer is less than the expected one. *)
    ippStsStreamEnd           = 34,      (* DC: The end of stream processed. *)
    ippStsDoubleSize        =   35,      (* Width or height of image is odd. *)
    ippStsNotSupportedCpu   =   36,      (* The CPU is not supported. *)
    ippStsUnknownCacheSize  =   37,      (* The CPU is supported, but the size of the cache is unknown. *)
    ippStsSymKernelExpected =   38,      (* The Kernel is not symmetric. *)
    ippStsEvenMedianWeight  =   39,      (* Even weight of the Weighted Median Filter is replaced with the odd one. *)
    ippStsWrongIntersectVOI =   40,      (* VOI has no intersection with the source or destination volume. No operation.                            *)
    ippStsI18nMsgCatalogInvalid=41,      (* Message Catalog is invalid, English message returned.                                                    *)
    ippStsI18nGetMessageFail  = 42,      (* Failed to fetch a localized message, English message returned. For more information use errno on Linux* OS and GetLastError on Windows* OS. *)
    ippStsWaterfall           = 43,      (* Cannot load required library, waterfall is used. *)
    ippStsPrevLibraryUsed     = 44,      (* Cannot load required library, previous dynamic library is used. *)
    ippStsLLADisabled         = 45,      (* OpenMP* Low Level Affinity is disabled. *)
    ippStsNoAntialiasing      = 46,      (* The mode does not support antialiasing. *)
    ippStsRepetitiveSrcData   = 47,      (* DC: The source data is too repetitive. *)
    ippStsSizeWrn             = 48,      (* The size does not allow to perform full operation. *)
    ippStsFeatureNotSupported = 49,      (* Current CPU doesn't support at least 1 of the desired features. *)
    ippStsUnknownFeature      = 50,      (* At least one of the desired features is unknown. *)
    ippStsFeaturesCombination = 51,      (* Wrong combination of features. *)
    ippStsAccurateModeNotSupported = 52  (* Accurate mode is not supported. *)
  );


const
  AverageKernel_3x3_16s: array[0..8] of Ipp16s = (1,1,1,1,1,1,1,1,1);
  Neighbors4_3x3_8u: array[0..8] of Ipp8u = (0,1,0,1,0,1,0,1,0);
  Neighbors8_3x3_8u: array[0..8] of Ipp8u = (1,1,1,1,0,1,1,1,1);
  Point00: ippiPoint = (x:0; y:0);
  Point11: ippiPoint = (x:1; y:1);

{$IFNDEF LOAD_DYNAMICALLY}
(******************************************************************************
 *                                                                            *
 *                          Core (ippcore.h)                                  *
 *                                                                            *
 ******************************************************************************)

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Automatic switching to best for current cpu library code using.
//
//  Notes:      At the moment of this function execution no any other Intel(R) IPP function
//              has to be working
*)
function ippInit: IppStatus; extdecl; external libippcore;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    getting of the library version
//  Returns:    the structure of information about version of ippcore library
//
//  Notes:      not necessary to release the returned structure
*)
function ippGetLibVersion: PIppLibraryVersion; extdecl; external libippcore;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: Changes the set of enabled/disabled CPU features.
//           This function sets the processor-specific code of the Intel(R) IPP
//           library according to the processor features specified in cpuFeatures.
//
*)
function ippSetCpuFeatures(cpuFeatures: Ipp64u): IppStatus; extdecl; external libippcore;


(******************************************************************************
 *                                                                            *
 *                     Signal Processing (ipps.h)                             *
 *                                                                            *
 ******************************************************************************)

(* /////////////////////////////////////////////////////////////////////////////
//                   Functions to allocate and free memory
///////////////////////////////////////////////////////////////////////////// *)
(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Aligned memory allocation (alignment depends on
//              CPU-architecture, 64-byte (cache line size) in most cases)
//  Parameter:
//    len       Number of elements (according to their type)
//  Returns:    Pointer to allocated memory, NULL in case of failure or len < 1
//
//  Notes:      Memory allocated by ippsMalloc must be freed by ippsFree
//              function only.
*)
function ippsMalloc_8u(len: int): PIpp8u; extdecl; external libipps;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Free memory allocated by ippsMalloc function
//  Notes:      Use this function to free memory allocated by ippsMalloc_* only
*)
procedure ippsFree(ptr: Pointer); extdecl; external libipps;

(* ////////////////////////////////////////////////////////////////////////////
//  Purpose:     Subtracts each element of vector from constant value
*)
function ippsSubCRev_8u_ISfs(val: Ipp8u; pSrcDst: PIpp8u; len: int; scaleFactor: int): IppStatus;
    extdecl; external libipps;
function ippsSubCRev_8u_Sfs(pSrc: PIpp8u; val: Ipp8u; pDst: PIpp8u; len: int; scaleFactor: int): IppStatus;
    extdecl; external libipps;


(******************************************************************************
 *                                                                            *
 *                      Image Processing (ippi.h)                             *
 *                                                                            *
 ******************************************************************************)

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    gets the version of the library
//  Notes:      there is no need to release the returned structure
*)
function ippiGetLibVersion: PIppLibraryVersion; extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                   Memory Allocation Functions
///////////////////////////////////////////////////////////////////////////// *)
(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    allocates memory with 32-byte aligned pointer for ippIP images,
//              every line of the image is aligned due to the padding characterized
//              by pStepBytes
//  Parameter:
//    widthPixels   width of image in pixels
//    heightPixels  height of image in pixels
//    pStepBytes    pointer to the image step, it is an output parameter calculated by the function
//
//  Returns:    pointer to the allocated memory or NULL if out of memory or wrong parameters
//  Notes:      free the allocated memory using the function ippiFree only
*)
function ippiMalloc_8u_C1 (widthPixels, heightPixels: int; var StepBytes: int): PIpp8u;  extdecl; external libippi;
function ippiMalloc_16u_C1(widthPixels, heightPixels: int; var StepBytes: int): PIpp16u; extdecl; external libippi;
function ippiMalloc_16s_C1(widthPixels, heightPixels: int; var StepBytes: int): PIpp16s; extdecl; external libippi;
function ippiMalloc_32f_C1(widthPixels, heightPixels: int; var StepBytes: int): PIpp32f; extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiFree
//  Purpose:    frees memory allocated by the ippiMalloc functions
//  Parameter:
//    ptr       pointer to the memory allocated by the ippiMalloc functions
//
//  Notes:      use this function to free memory allocated by ippiMalloc
*)
procedure ippiFree(ptr: Pointer); extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  Arithmetic Functions
///////////////////////////////////////////////////////////////////////////// *)
function ippiAdd_8u_C1IRSfs(pSrc: PIpp8u; srcStep: int; pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; scaleFactor: int): IppStatus;
    extdecl; external libippi;
function ippiSub_8u_C1IRSfs(pSrc: PIpp8u; srcStep: int; pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; scaleFactor: int): IppStatus;
    extdecl; external libippi;
function ippiAdd_8u_C1RSfs(pSrc1: PIpp8u; src1Step: int; pSrc2: PIpp8u; src2Step: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; scaleFactor: int): IppStatus;
    extdecl; external libippi;
function ippiSub_8u_C1RSfs(pSrc1: PIpp8u; src1Step: int; pSrc2: PIpp8u; src2Step: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; scaleFactor: int): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  FFT Transforms
///////////////////////////////////////////////////////////////////////////// *)
function ippiFFTGetSize_R_32f(orderX, orderY: int; flag: int; hint: IppHintAlgorithm; var pSizeSpec, pSizeInit, pSizeBuf: int): IppStatus;
    extdecl; external libippi;
function ippiFFTInit_R_32f(orderX, orderY: int; flag: int; hint: IppHintAlgorithm; pFFTSpec: PIppiFFTSpec_R_32f; pMemInit: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFFTFwd_RToPack_32f_C1IR(pSrcDst: PIpp32f; srcDstStep: int; pFFTSpec: PIppiFFTSpec_R_32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  DFT Transforms
///////////////////////////////////////////////////////////////////////////// *)
function ippiDFTGetSize_R_32f(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; var pSizeSpec: int; var pSizeInit: int; var pSizeBuf: int): IppStatus;
    extdecl; external libippi;
function ippiDFTGetSize_C_32fc(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; var pSizeSpec, pSizeInit, pSizeBuf: int): IppStatus;
    extdecl; external libippi;
function ippiDFTInit_R_32f(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; pDFTSpec: PIppiDFTSpec_R_32f; pMemInit: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiDFTInit_C_32fc(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; pDFTSpec: PIppiDFTSpec_C_32fc; pMemInit: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiDFTFwd_RToPack_32f_C1IR(pSrcDst: PIpp32f; srcDstStep: int; pDFTSpec: PIppiDFTSpec_R_32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiDFTFwd_CToC_32fc_C1IR(pSrcDst: PIpp32fc; srcDstStep: int; pDFTSpec: PIppiDFTSpec_C_32fc; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

function ippiPhasePackGetBufferSize_32f(numChannels: int; dstRoiSize: IppiSize; var pSize: int): IppStatus;
    extdecl; external libippi;
function ippiMagnitudePackGetBufferSize_32f(numChannels: int; dstRoiSize: IppiSize; var pSize: int): IppStatus;
    extdecl; external libippi;
function ippiPhasePack_32f_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; dstRoiSize: IppiSize; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiMagnitudePack_32f_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; dstRoiSize: IppiSize; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiPhase_32fc32f_C1R(pSrc: PIpp32fc; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiMagnitude_32fc32f_C1R(pSrc: PIpp32fc; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  Convert and Initialization functions
///////////////////////////////////////////////////////////////////////////// *)

(* ////////////////////////////////////////////////////////////////////////////
//  Name:       ippiCopy
//
//  Purpose:  copy pixel values from the source image to the destination  image
//
*)
function ippiCopy_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize): IppStatus; extdecl; external libippi;
function ippiCopy_32f_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int;
    roiSize: IppiSize): IppStatus; extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Name:       ippiCopyReplicateBorder
//
//  Purpose:   Copies pixel values between two buffers and adds
//             the replicated border pixels.
//
//  Parameters:
//    pSrc                Pointer  to the source image buffer
//    srcStep             Step in bytes through the source image
//    pDst                Pointer to the  destination image buffer
//    dstStep             Step in bytes through the destination image
//    scrRoiSize          Size of the source ROI in pixels
//    dstRoiSize          Size of the destination ROI in pixels
//    topBorderHeight     Height of the top border in pixels
//    leftBorderWidth     Width of the left border in pixels
*)
function ippiCopyReplicateBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize;
    pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int): IppStatus;
    extdecl; external libippi;
function ippiCopyReplicateBorder_8u_C1IR(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize;
    dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int): IppStatus;
    extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Name:       ippiCopyConstBorder
//
//  Purpose:    Copies pixel values between two buffers and adds
//              the border pixels with constant value.
*)
function ippiCopyConstBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize;
    pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int; value: Ipp8u): IppStatus;
    extdecl; external libippi;
function ippiCopyConstBorder_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int; srcRoiSize: IppiSize;
    dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int; value: Ipp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiSet
//
//  Purpose:    Sets pixels in the image buffer to a constant value
//
*)
function ippiSet_8u_C1R(value: Ipp8u; pDst: PIpp8u; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiSet_16u_C1R(value: Ipp16u; pDst: PIpp16u; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Name:       ippiConvert
//
//  Purpose:    Converts pixel values of an image from one bit depth to another
//
//  The result of integer operations is always saturated to the destination data type range.
//  It means that if the value of the source pixel is out of the data range of the destination image, the value of the corresponding destination pixel
//  is set to the value of the lower or upper bound (minimum or maximum) of the destination data range
*)
function ippiConvert_8u16u_C1R(pSrc: PIpp8u;  srcStep: int; pDst: PIpp16u; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_8u16s_C1R(pSrc: PIpp8u;  srcStep: int; pDst: PIpp16s; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_8u32s_C1R(pSrc: PIpp8u;  srcStep: int; pDst: PIpp32s; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_8u32f_C1R(pSrc: PIpp8u;  srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_16u32s_C1R(pSrc: PIpp16u; srcStep: int; pDst: PIpp32s; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_16u8u_C1R(pSrc: PIpp16u; srcStep: int; pDst: PIpp8u;  dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_16s8u_C1R(pSrc: PIpp16s; srcStep: int; pDst: PIpp8u;  dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiConvert_32f8u_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp8u;  dstStep: int; roiSize: IppiSize; roundMode: IppRoundMode): IppStatus;
    extdecl; external libippi;
function ippiConvert_32f32s_C1RSfs(pSrc: PIpp32f; srcStep: int; pDst: PIpp32s; dstStep: int; roi: IppiSize; round: IppRoundMode; scaleFactor: int): IppStatus;
    extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
// Name:        ippiScaleC
//
// Purpose:     Converts data with scaling by formula: dst = src*mVal + aVal
//
*)
function ippiScaleC_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int; mVal, aVal: Ipp64f;
    roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus;
    extdecl; external libippi;
function ippiScaleC_8u_C1R(pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus;
    extdecl; external libippi;
function ippiScaleC_8u16s_C1R(pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp16s; dstStep: int;
    roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus;
    extdecl; external libippi;
function ippiScaleC_8u32f_C1R(pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp32f; dstStep: int;
    roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  Logical Operations and Shift Functions
///////////////////////////////////////////////////////////////////////////// *)
function ippiXorC_8u_C1IR(value: Ipp8u; pSrcDst: PIpp8u; srcDstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiXorC_8u_C1R(pSrc: PIpp8u; srcStep: int; value: Ipp8u; pDst: PIpp8u; dstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;
function ippiXorC_16u_C1IR(value: Ipp16u; pSrcDst: PIpp16u; srcDstStep: int; roiSize: IppiSize): IppStatus;
    extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//                           Compare Operations
///////////////////////////////////////////////////////////////////////////// *)
(* /////////////////////////////////////////////////////////////////////////////
//  Name:           ippiCompare
//                  ippiCompareC
//  Purpose:  Compares pixel values of two images, or pixel values of an image to a constant
//            If the result of the compare is true, then the corresponding output pixel is set to an IPP_MAX_8U value; otherwise, it is set to 0
*)
function ippiCompareC_8u_C1R(pSrc: PIpp8u; srcStep: int; value: Ipp8u;
    pDst: PIpp8u; dstStep: int; roiSize: IppiSize; ippCmpOp: IppCmpOp): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:   Get the sizes (in bytes) of the spec and the buffer for ippiHistogram_.
*)
function ippiHistogramGetBufferSize(dataType: IppDataType; roiSize: IppiSize; nLevels: Pint; numChannels: int; uniform: int; var SpecSize: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiHistogramUniformInit(dataType: IppDataType; lowerLevel, upperLevel: PIpp32f; nLevels: Pint; numChannels: int; pSpec: PIppiHistogramSpec): IppStatus;
    extdecl; external libippi;
function ippiHistogram_8u_C1R(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; pHist: PIpp32u; pSpec: PIppiHistogramSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                   Threshold operations
///////////////////////////////////////////////////////////////////////////// *)

(* ////////////////////////////////////////////////////////////////////////////
//  Purpose:   Performs threshold operation using the comparison "greater than"
//             If the result of the compare is true, the corresponding output pixel is set to the threshold value.
//             Otherwise, it is set to the source pixel value.
//
//  Parameters:
//   pSrc       Pointer to the source image
//   srcStep    Step through the source image
//   pDst       Pointer to the destination image
//   dstStep    Step through the destination image
//   pSrcDst    Pointer to the source/destination image (in-place flavors)
//   srcDstStep Step through the source/destination image (in-place flavors)
//   roiSize    Size of the ROI
//   threshold  Threshold level value (array of values for multi-channel data)
*)
function ippiThreshold_GT_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; threshold: Ipp8u): IppStatus; extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Purpose:  Performs thresholding of pixel values: pixels that are
//            less/greater than threshold, are set to a specified value
//
//  Parameters:
//   pSrc       Pointer to the source image
//   srcStep    Step through the source image
//   pDst       Pointer to the destination image
//   dstStep    Step through the destination image
//   pSrcDst    Pointer to the source/destination image (in-place flavors)
//   srcDstStep Step through the source/destination image (in-place flavors)
//   roiSize    Size of the ROI
//   threshold  Threshold level value (array of values for multi-channel data)
//   value      The output value (array or values for multi-channel data)
*)
function ippiThreshold_LTVal_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; threshold: Ipp8u; value: Ipp8u): IppStatus; extdecl; external libippi;
function ippiThreshold_LTVal_16u_C1IR(pSrcDst: PIpp16u; srcDstStep: int;
    roiSize: IppiSize; threshold: Ipp16u; value: Ipp16u): IppStatus; extdecl; external libippi;
function ippiThreshold_LTVal_32f_C1IR(pSrcDst: PIpp32f; srcDstStep: int ;
    roiSize: IppiSize; threshold: Ipp32f; value: Ipp32f): IppStatus; extdecl; external libippi;
function ippiThreshold_GTVal_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; threshold: Ipp8u; value: Ipp8u): IppStatus; extdecl; external libippi;
function ippiThreshold_GTVal_16u_C1IR(pSrcDst: PIpp16u; srcDstStep: int;
    roiSize: IppiSize; threshold: Ipp16u; value: Ipp16u): IppStatus; extdecl; external libippi;
//function ippiThreshold_GTVal_8u_C1R(pSrc: PIpp8u; srcStep:int; pDst: PIpp8u; dstStep: int;
//    roiSize: IppiSize; threshold: Ipp8u; value: Ipp8u): IppStatus; extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Purpose:    Performs double thresholding of pixel values
//
//  Parameters:
///  Parameters:
//   pSrc        Pointer to the source image
//   srcStep     Step through the source image
//   pDst        Pointer to the destination image
//   dstStep     Step through the destination image
//   pSrcDst     Pointer to the source/destination image (in-place flavors)
//   srcDstStep  Step through the source/destination image (in-place flavors)
//   roiSize     Size of the ROI
//   thresholdLT Lower threshold value (array of values for multi-channel data)
//   valueLT     Lower output value (array or values for multi-channel data)
//   thresholdGT Upper threshold value (array of values for multi-channel data)
//   valueGT     Upper output value (array or values for multi-channel data)
*)
function ippiThreshold_LTValGTVal_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int;
    roiSize: IppiSize; thresholdLT: Ipp8u; valueLT: Ipp8u; thresholdGT: Ipp8u; valueGT: Ipp8u): IppStatus;
    extdecl; external libippi;

(* ////////////////////////////////////////////////////////////////////////////
//  Purpose: Calculate Otsu theshold value of images
//
//    Parameters:
//      pSrc                     Pointer to image
//      srcStep                  Image step
//      roiSize                  Size of image ROI
//      pThreshold               Returned Otsu theshold value
*)
function ippiComputeThreshold_Otsu_8u_C1R(pSrc: PIpp8u; srcStep: int;
    roiSize: IppiSize; var Threshold: Ipp8u): IppStatus;
    extdecl; external libippi;


(* /////////////////////////////////////////////////////////////////////////////
//                     Adaptive threshold functions
// /////////////////////////////////////////////////////////////////////////////
//  Purpose: to define spec and buffer sizes for adaptive threshold
*)
function ippiThresholdAdaptiveBoxGetBufferSize(roiSize: IppiSize; maskSize: IppiSize; dataType: IppDataType;
    numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;

//     Output pixels are calculated such:
//     pDst(x,y) = valGT if (pSrc(x,y) > T(x,y))
//     and
//     pDst(x,y) = valLE if (pSrc(x,y) <= T(x,y))
//
//     For ippiThresholdAdaptiveBox_8u_C1R:
//     T(x,y) is a mean of the kernelSize.width*kernelSize.height neighborhood of (x,y)-pixel
//     minus delta.
function ippiThresholdAdaptiveBox_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize; maskSize: IppiSize;
    delta: Ipp32f; valGT, valLE: Ipp8u; borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiThresholdAdaptiveBox_8u_C1IR(pSrcDst: PIpp8u; srcDstStep: int; roiSize: IppiSize; maskSize: IppiSize;
    delta: Ipp32f; valGT, valLE: Ipp8u; borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: Perform deconvolution for source image using Lucy-Richardson algorithm
*)
function ippiDeconvLRGetSize_32f(numChannels, kernelSize: int; maxRoi: IppiSize; var Size: int): IppStatus; extdecl;
    extdecl; external libippi;
function ippiDeconvLRInit_32f_C1R(pDeconvLR: PIppiDeconvLR_32f_C1R; pKernel: PIpp32f; kernelSize: int; maxRoi: IppiSize; threshold: Ipp32f): IppStatus;
     extdecl; external libippi;
function ippiDeconvLR_32f_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize; numIter: int; pDeconvLR: PIppiDeconvLR_32f_C1R): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                      Resize Transform Functions
///////////////////////////////////////////////////////////////////////////// *)
(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: Computes the size of Spec structure and temporal buffer for Resize transform
//
*)
function ippiResizeGetSize_8u(srcSize: IppiSize; dstSize: IppiSize; interpolation: IppiInterpolationType;
    antialiasing: Ipp32u; var SpecSize, InitBufSize: int): IppStatus;
    extdecl; external libippi;

function ippiResizeGetBufferSize_8u(pSpec: PIppiResizeSpec_32f; dstSize: IppiSize; numChannels: Ipp32u; var BufSize: int): IppStatus;
    extdecl; external libippi;

function ippiResizeNearestInit_8u(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus;
    extdecl; external libippi;
function ippiResizeLinearInit_8u(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus;
    extdecl; external libippi;
function ippiResizeSuperInit_8u(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus;
    extdecl; external libippi;

function ippiResizeNearest_8u_C1R(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
    dstOffset: IppiPoint; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiResizeLinear_8u_C1R(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
    dstOffset: IppiPoint; dstSize: IppiSize; border: IppiBorderType; pBorderValue: PIpp8u;
    pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiResizeSuper_8u_C1R(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
    dstOffset: IppiPoint; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;


(* /////////////////////////////////////////////////////////////////////////////
//                     Affine Transform functions
// ////////////////////////////////////////////////////////////////////////// *)

(* /////////////////////////////////////////////////////////////////////////////
//  Name:               ippiGetAffineBound
//  Purpose:            Computes the bounding rectangle of the transformed image ROI
*)
function ippiGetAffineBound(srcROI: IppiRect; var bound: array_double_2_2; const coeffs: array_double_2_3): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:               ippiGetAffineQuad
//  Purpose:            Computes coordinates of the quadrangle to which a source ROI is mapped
*)
function ippiGetAffineQuad(srcROI: IppiRect; var quad: array_double_4_2; const coeffs: array_double_2_3): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:            Calculates shifts for ippiGetRotateTransform function to rotate an image
//                      around the specified center (xCenter, yCenter)
//  Parameters:
//    xCenter, yCenter  Coordinates of the center of rotation
//    angle             The angle of clockwise rotation, degrees
//    xShift, yShift    Pointers to the shift values
*)
function ippiGetRotateShift(xCenter, yCenter: double; angle: double; var xShift, yShift: double): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:               Computes the affine coefficients for the transform that
//                         rotates an image around (0, 0) by specified angle + shifts it
//                         | cos(angle)  sin(angle)  xShift|
//                         |                               |
//                         |-sin(angle)  cos(angle)  yShift|
*)
function ippiGetRotateTransform(angle: double; xShift, yShift: double; var coeffs: array_double_2_3): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:            Computes the size of external buffer for Warp transform
*)
function ippiWarpGetBufferSize(pSpec: PIppiWarpSpec; dstRoiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:               ippiWarpAffineGetSize
//  Purpose:            Computes the size of Spec structure and temporal buffer for Affine transform
*)
function ippiWarpAffineGetSize(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
    coeffs: array_double_2_3; interpolation: IppiInterpolationType; direction: IppiWarpDirection;
    borderType: IppiBorderType; var SpecSize: int; var InitBufSize: int): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:            Initializes the Spec structure for the Warp affine transform
//                      by different interpolation methods
*)
function ippiWarpAffineNearestInit(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
    coeffs: array_double_2_3; direction: IppiWarpDirection; numChannels: int; borderType: IppiBorderType;
    pBorderValue: PIpp64f; smoothEdge: int; pSpec: PIppiWarpSpec): IppStatus;
    extdecl; external libippi;
function ippiWarpAffineLinearInit(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
    coeffs: array_double_2_3; direction: IppiWarpDirection; numChannels: int; borderType: IppiBorderType;
    pBorderValue: PIpp64f; smoothEdge: int; pSpec: PIppiWarpSpec): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:            Performs affine transform of an image with using different interpolation methods
*)
function ippiWarpAffineNearest_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    dstRoiOffset: IppiPoint; dstRoiSize: IppiSize; pSpec: PIppiWarpSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiWarpAffineLinear_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    dstRoiOffset: IppiPoint; dstRoiSize: IppiSize; pSpec: PIppiWarpSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                   Statistic functions
///////////////////////////////////////////////////////////////////////////// *)

(* /////////////////////////////////////////////////////////////////////////////
//  Name:      ippiMoments
//
//  Purpose:   Computes statistical moments of an image
//
//    mOrd        m- Order (X direction)
//    nOrd        n- Order (Y direction)
//    nChannel    Channel number
//    roiOffset   Offset of the ROI origin (ippiGetSpatialMoment ONLY!)
*)
function ippiMomentGetStateSize_64f(hint: IppHintAlgorithm; var pSize: int): IppStatus;
    extdecl; external libippi;
function ippiMomentInit_64f(pState: PIppiMomentState_64f; hint: IppHintAlgorithm): IppStatus;
    extdecl; external libippi;
function ippiMoments64f_8u_C1R(pSrc: PIpp8u; srcStep:int; roiSize: IppiSize; pCtx: PIppiMomentState_64f): IppStatus;
    extdecl; external libippi;
function ippiGetSpatialMoment_64f(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel:int; roiOffset: IppiPoint; var pValue: Ipp64f): IppStatus;
    extdecl; external libippi;
function ippiGetCentralMoment_64f(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel: int; var pValue: Ipp64f): IppStatus;
    extdecl; external libippi;
function ippiGetNormalizedCentralMoment_64f(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel: int; var pValue: Ipp64f): IppStatus;
    extdecl; external libippi;
function ippiGetHuMoments_64f(pState: PIppiMomentState_64f; nChannel: int; pHm: PIppiHuMoment_64f): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiFilterBorderGetSize, ippiFilterBorderInit, ippiFilterBorder
//  Purpose:    Filters an image using a general integer rectangular kernel
*)
function ippiFilterBorderGetSize(kernelSize: IppiSize; dstRoiSize: IppiSize; dataType: IppDataType;
    kernelType: IppDataType; numChannels: int; var SpecSize: int; var BufferSize:int): IppStatus;
    extdecl; external libippi;

function ippiFilterBorderInit_16s(pKernel: PIpp16s; kernelSize: IppiSize; divisor: int; dataType: IppDataType;
    numChannels: int; roundMode: IppRoundMode; pSpec: PIppiFilterBorderSpec): IppStatus;
    extdecl; external libippi;
function ippiFilterBorderInit_32f(pKernel: PIpp32f; kernelSize: IppiSize; dataType: IppDataType;
    numChannels: int; roundMode: IppRoundMode; pSpec: PIppiFilterBorderSpec): IppStatus;
    extdecl; external libippi;

function ippiFilterBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    dstRoiSize: IppiSize; border: IppiBorderType; borderValue: PIpp8u;
    pSpec: PIppiFilterBorderSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFilterBorder_32f_C1R(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int;
    dstRoiSize: IppiSize; border: IppiBorderType; borderValue: PIpp32f;
    pSpec: PIppiFilterBorderSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:               ippiFilterMedianBorderGetBufferSize
//
//  Purpose:            Computes the size of the external buffer for median filter with border
//
//  Parameters:
//   roiSize            Size of destination ROI in pixels.
//   maskSize           Size of filter mask.
//   dataType           Data type of the source an desination images.
//   numChannels        Number of channels in the images. Possible values are 1, 3 or 4.
//   pBufferSize        Pointer to the size (in bytes) of the external work buffer.
//
*)
function ippiFilterMedianBorderGetBufferSize(dstRoiSize: IppiSize; maskSize: IppiSize;
    dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:            Perform median filtering of an image with border
//
//  Parameters:
//   pSrc               Pointer to the source image ROI.
//   srcStep            Distance in bytes between starting points of consecutive lines in the sorce image.
//   pDst               Pointer to the destination image ROI.
//   dstStep            Distance in bytes between starting points of consecutive lines in the destination image.
//   dstRoiSize         Size of destination ROI in pixels.
//   maskSize           Size of filter mask.
//   borderType         Type of border.
//   borderValue        Constant value to assign to pixels of the constant border. This parameter is applicable
//                      only to the ippBorderConst border type.
//   pBorderValue       Pointer to constant value to assign to pixels of the constant border. This parameter is applicable
//                      only to the ippBorderConst border type.
//   pBuffer            Pointer to the work buffer.
*)
function ippiFilterMedianBorder_8u_C1R(pSrc: PIpp8u; srcStep: int;
    pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; maskSize: IppiSize;
    borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;


(* ///////////////////////////////////////////////////////////////////////////
//             Filters with Fixed Kernel
/////////////////////////////////////////////////////////////////////////// *)
(* ////////////////////////////////////////////////////////////////////////////
//              Kernels:
//
//                                1  1  1
//              PrewittHoriz      0  0  0
//                               -1 -1 -1
//
//
//                               -1  0  1
//              PrewittVert      -1  0  1
//                               -1  0  1
//
//
//                                1  2  1
//              SobelHoriz        0  0  0
//                               -1 -2 -1
//
//
//                               -1  0  1
//              SobelVert        -2  0  2
//                               -1  0  1
//
//
//                               -1 -1 -1
//              Sharpen          -1 16 -1  X  1/8
//                               -1 -1 -1
//
//
//                               -1 -1 -1
//              Laplace (3x3)    -1  8 -1
//                               -1 -1 -1
//
//
//                                1  2  1
//              Gauss (3x3)       2  4  2  X  1/16
//                                1  2  1
//
//
//                                2   7  12   7   2
//                                7  31  52  31   7
//              Gauss (5x5)      12  52 127  52  12  X  1/571
//                                7  31  52  31   7
//                                2   7  12   7   2
///////////////////////////////////////////////////////////////////////////// *)

(* /////////////////////////////////////////////////////////////////////////////
//
//  Purpose:            Computes the size of the external buffer for fixed filter with border
//
*)
function ippiFilterPrewittHorizBorderGetBufferSize(dstRoiSize: IppiSize; mask: IppiMaskSize;
    srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiFilterPrewittVertBorderGetBufferSize (dstRoiSize: IppiSize; mask: IppiMaskSize;
    srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiFilterLaplaceBorderGetBufferSize(dstRoiSize: IppiSize; mask: IppiMaskSize;
    srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiFilterSobelGetBufferSize(dstRoiSize: IppiSize; mask: IppiMaskSize; normType: IppNormType;
    srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiFilterSharpenBorderGetBufferSize(dstRoiSize: IppiSize; mask: IppiMaskSize;
    srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
//
//  Purpose:            Perform linear filtering of an image using one of
//                      predefined convolution kernels.
function ippiFilterPrewittVertBorder_8u16s_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
    borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFilterPrewittHorizBorder_8u16s_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
    borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFilterLaplaceBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    dstRoiSize: IppiSize; mask: IppiMaskSize; borderType: IppiBorderType; borderValue: Ipp8u;
    pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFilterSobel_8u16s_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType; borderType: IppiBorderType;
    borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiFilterSharpenBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
    borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
//                  Wiener Filters
///////////////////////////////////////////////////////////////////////////// *)
//  Purpose: Performs two-dimensional adaptive noise-removal
//           filtering of an image using Wiener filter.
function ippiFilterWienerGetBufferSize(dstRoiSize: IppiSize; maskSize: IppiSize; channels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiFilterWiener_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; maskSize: IppiSize; anchor: IppiPoint; noise: PIpp32f; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
// Purpose: Computes the size of the external buffer for
//          Gradient() calls
*)
function ippiGradientVectorGetBufferSize(roiSize: IppiSize; mask: IppiMaskSize;
    dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippi;
(* /////////////////////////////////////////////////////////////////////////////
// Purpose: Computes gradient vectors over an image using Sobel, Scharr or Prewitt operator
*)
function ippiGradientVectorPrewitt_8u16s_C1R(pSrc: PIpp8u; srcStep: int;
    pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
    pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
    borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiGradientVectorSobel_8u16s_C1R(pSrc: PIpp8u; srcStep: int;
    pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
    pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
    borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;
function ippiGradientVectorScharr_8u16s_C1R(pSrc: PIpp8u; srcStep: int;
    pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
    pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
    dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
    borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
// Purpose: Computes HOG descriptor
*)
function ippiHOGGetSize(pConfig: PIppiHOGConfig; var pHOGSpecSize: int): IppStatus;
    extdecl; external libippi;
function ippiHOGInit(pConfig: PIppiHOGConfig; pHOGSpec: PIppiHOGSpec): IppStatus;
    extdecl; external libippi;
function ippiHOGGetBufferSize(pHOGSpec: PIppiHOGSpec; roiSize: IppiSize; var pBufferSize: int): IppStatus;
    extdecl; external libippi;
function ippiHOGGetDescriptorSize(pHOGSpec: PIppiHOGSpec; var pWinDescriptorSize: int): IppStatus;
    extdecl; external libippi;
function ippiHOG_8u32f_C1R(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; pLocation: PIppiPoint; nLocations: int; pDst: PIpp32f; pHOGSpec: PIppiHOGSpec; borderID: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippi;

(* /////////////////////////////////////////////////////////////////////////////
// Purpose: Calculates the LBP of the image.
*)
function ippiLBPImageMode3x3_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; mode: int; borderType: IppiBorderType; borderValue: PIpp8u): IppStatus;
    extdecl; external libippi;


(******************************************************************************
 *                                                                            *
 *                      Computer Vision (ippcv.h)                             *
 *                                                                            *
 ******************************************************************************)

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Draw bounds between image segments
*)
function ippiSegmentWatershedGetBufferSize_8u16u_C1R(roiSize: IppiSize; var pBufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiSegmentWatershed_8u16u_C1IR(pSrc: PIpp8u; srcStep: int; pMarker: PIpp16u; markerStep: int; roiSize: IppiSize; norm: IppiNorm; flag: int; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiBoundSegments_16u_C1IR(pMarker: PIpp16u; markerStep: int; roiSize: IppiSize; val: Ipp16u; norm: IppiNorm): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Labels connected non-zero components with different label values
*)
function ippiLabelMarkersGetBufferSize_8u32s_C1R(roiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiLabelMarkersGetBufferSize_16u_C1R(roiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiLabelMarkers_8u32s_C1R(pSrcMarker: PIpp8u; srcMarkerStep: int; pDstMarker: PIpp32s; dstMarkerStep: int;
    roiSize: IppiSize; minLabel, maxLabel: int; norm: IppiNorm; var Number: int; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiLabelMarkers_16u_C1IR(pMarker: PIpp16u; markerStep: int; roiSize: IppiSize;
    minLabel, maxLabel: int; norm: IppiNorm; var Number: int; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiFilterGaussianGetBufferSize
//
//  Purpose:    Computes the size of the working buffer for the Gaussian filter
//
//  Arguments:
//     maxRoiSize           Maximal size of the image ROI in pixels.
//     kernelSize           Size of the Gaussian kernel (odd, greater or equal to 3).
//     dataType             Data type of the source and destination images.
//     numChannels          Number of channels in the images. Possible values are 1 and 3.
//     pSpecSize            Pointer to the computed size (in bytes) of the Gaussian  specification structure.
//     pBufferSize          Pointer to the computed size (in bytes) of the external buffer.
*)
function ippiFilterGaussianGetBufferSize(maxRoiSize: IppiSize; kernelSize: Ipp32u;
    dataType: IppDataType; numChannels: int; var SpecSize: int; var BufferSize: int): IppStatus;
    extdecl; external libippcv;


(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiFilterGaussianInit
//
//  Purpose:    initialization of Spec for Gaussian filter
//
//  Arguments:
//     roiSize              Size of the image ROI in pixels.
//     kernelSize           Size of the Gaussian kernel (odd, greater or equal to 3).
//     sigma                Standard deviation of the Gaussian kernel.
//     borderType           One of border supported types.
//     dataType             Data type of the source and destination images.
//     numChannels          Number of channels in the images. Possible values are 1 and 3.
//     pSpec                Pointer to the Spec.
//     pBuffer              Pointer to the buffer:
*)
function ippiFilterGaussianInit(roiSize: IppiSize; kernelSize: Ipp32u; sigma: Ipp32f;
    borderType: IppiBorderType; dataType: IppDataType; numChannels: int;
    Spec: PIppFilterGaussianSpec; Buffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Name:       ippiFilterGaussian
//
//  Purpose:    Applies Gaussian filter with borders
//
//  Arguments:
//     pSrc             Pointer to the source image ROI.
//     srcStep          Distance in bytes between starts of consecutive lines in the source image.
//     pDst             Pointer to the destination image ROI.
//     dstStep          Distance in bytes between starts of consecutive lines in the destination image.
//     roiSize          Size of the source and destination image ROI.
//     borderType       One of border supported types.
//     borderValue      Constant value to assign to pixels of the constant border. if border type equals ippBorderConstant
//     pSpec            Pointer to the Gaussian specification structure.
//     pBuffer          Pointer to the working buffer.
*)
function ippiFilterGaussianBorder_8u_C1R(pSrc: PIpp8u; srcStep: int;
    pDst: PIpp8u; dstStep: int; roiSize: IppiSize; borderValue: Ipp8u;
    Spec: PIppFilterGaussianSpec; Buffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(*******************************************************************************
*                              Distance Transform                              *
*******************************************************************************)
function ippiDistanceTransform_3x3_8u16u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp16u; dstStep:int; roiSize: IppiSize; pMetrics: PIpp32s): IppStatus;
    extdecl; external libippcv;
function ippiDistanceTransform_3x3_8u32f_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep:int; roiSize: IppiSize; pMetrics: PIpp32f): IppStatus;
    extdecl; external libippcv;
function ippiTrueDistanceTransformGetBufferSize_8u32f_C1R(roiSize: IppiSize; var BufferSize: int): IppStatus;
    extdecl; external libippcv;
function ippiTrueDistanceTransform_8u32f_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep:int; roiSize: IppiSize; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(*******************************************************************************
*                                  Flood Fill                                  *
*******************************************************************************)
(*//////////////////////////////////////////////////////////////////////////////
//  Purpose: The functions fill the seed pixel enewValirons inside which all pixel
//           values are equal to (first 4 funcs) or not far from each other (the others).
//  Parameters:
//    seed                     Coordinates of the seed point inside image ROI,
//    newVal                   Value to fill with for one-channel data,
//    pRegion                  Pointer to repainted region properties structure,
*)
function ippiFloodFillGetSize(roiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiFloodFillGetSize_Grad(roiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiFloodFill_4Con_8u_C1IR(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
    seed: IppiPoint; newVal: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiFloodFill_8Con_8u_C1IR(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
    seed: IppiPoint; newVal: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiFloodFill_Range4Con_8u_C1IR(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
    seed: IppiPoint; newVal: Ipp8u; minDelta, maxDelta: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiFloodFill_Range8Con_8u_C1IR(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
    seed: IppiPoint; newVal: Ipp8u; minDelta, maxDelta: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiFloodFill_Range8Con_16u_C1IR(pImage: PIpp16u; imageStep: int; roiSize: IppiSize;
    seed: IppiPoint; newVal: Ipp16u; minDelta, maxDelta: Ipp16u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(*******************************************************************************
*                           Morphological Operations                           *
*******************************************************************************)
(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:  Gets the size of the internal state or specification structure for morphological operations.
*)
function ippiMorphologyBorderGetSize_8u_C1R(roiSize: IppiSize; maskSize: IppiSize; var SpecSize: int; var BufferSize: int): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:  Initialize the internal state or specification structure for morphological operation.
*)
function ippiMorphologyBorderInit_8u_C1R(roiSize: IppiSize; pMask: PIpp8u; maskSize: IppiSize;
    pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Perform erosion/dilation of the image arbitrary shape structuring element.
*)
function ippiDilateBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize;
    borderType: IppiBorderType; borderValue: Ipp8u; pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiErodeBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize;
    borderType: IppiBorderType; borderValue: Ipp8u; pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* ///////////////////////////////////////////////////////////////////////////////////////
//  Purpose:  Gets the size of the internal state or specification structure for advanced morphological operations.
*)
function ippiMorphAdvGetSize_8u_C1R(roiSize: IppiSize; maskSize: IppiSize; var SpecSize: int; var BufferSize: int): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:  Initialize the internal state or specification structure for advanced morphological operations.
*)
function ippiMorphAdvInit_8u_C1R(roiSize: IppiSize; pMask: PIpp8u; maskSize: IppiSize;
    pMorphSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose:    Perform advanced morphologcal operations on the image arbitrary shape structuring element.
*)
function ippiMorphOpenBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiMorphCloseBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiMorphTophatBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiMorphBlackhatBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiMorphGradientBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;


(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: Perform Hough transform algorithm
*)
function ippiHoughLineGetSize_8u_C1R(roiSize: IppiSize; delta: IppPointPolar; maxLineCount: int; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiHoughLine_8u32f_C1R(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; delta: IppPointPolar; threshold: int;
       pLine: PIppPointPolar; maxLineCount: int; var LineCount: int; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
function ippiHoughLine_Region_8u32f_C1R(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize;
       pLine: PIppPointPolar; dstRoi: PIppPointPolar; maxLineCount: int; var LineCount: int; delta: IppPointPolar; threshold: int; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;


(*//////////////////////////////////////////////////////////////////////////////
//  Purpose: Perform convolution operation with fixed kernels 3x3 and 5x5 and creates binary image of source's image edges,
//           using derivatives of the first order.
*)
function ippiCannyBorderGetSize(roiSize: IppiSize; filterType: IppiDifferentialKernel;
    mask: IppiMaskSize; dataType: IppDataType; var BufferSize: int ): IppStatus;
    extdecl; external libippcv;
function ippiCannyBorder_8u_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
    roiSize: IppiSize; filterType: IppiDifferentialKernel; mask: IppiMaskSize;
    borderType: IppiBorderType; borderValue: Ipp8u;
    lowThresh, highThresh: Ipp32f; norm: IppNormType; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: Performs Harris corner detector operations
*)
function ippiHarrisCornerGetBufferSize(roiSize: IppiSize; filterMask: IppiMaskSize;
    avgWndSize: Ipp32u; dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus;
    extdecl; external libippcv;
function ippiHarrisCorner_8u32f_C1R(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize;
    filterType: IppiDifferentialKernel; filterMask: IppiMaskSize; avgWndSize: Ipp32u; k, scale: float;
    borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;

(* /////////////////////////////////////////////////////////////////////////////
//  Purpose: corner finding FastN
//  Parameters:
//    circleRadius      Radius for corner finding. Possible values are 1, 3, 5, 7, 9.
//    N                 Critical number of pixels that have different value.
//    threshold         the level for definition of critical pixels
*)
function ippiFastNGetSize(srcSize: IppiSize; circleRadius, N, orientationBins, option: int;
    dataType: IppDataType; nChannels: int; var pSpecSize: int): IppStatus;
    extdecl; external libippcv;
function ippiFastNInit(srcSize: IppiSize; circleRadius, N, orientationBins, option: int;
    threshold: Ipp32f; dataType: IppDataType; nChannels: int; pSpec: PIppiFastNSpec): IppStatus;
    extdecl; external libippcv;
function ippiFastNGetBufferSize(pSpec: PIppiFastNSpec; dstRoiSize: IppiSize; var BufSize: int): IppStatus;
    extdecl; external libippcv;
function ippiFastN_8u_C1R(pSrc: PIpp8u; srcStep: int; pDstCorner: PIpp8u; dstCornerStep: int;
    pDstScore: PIpp8u; dstScoreStep: int; pNumCorner: pint; srcRoiOffset: IppiPoint; dstRoiSize: IppiSize;
    pSpec: PIppiFastNSpec; pBuffer: PIpp8u): IppStatus;
    extdecl; external libippcv;
{$ELSE}
var
  ippInit: function: IppStatus; extdecl;
  ippGetLibVersion: function: PIppLibraryVersion; extdecl;
  ippSetCpuFeatures: function (cpuFeatures: Ipp64u): IppStatus; extdecl;

  // Signal Processing (ipps.h)
  ippsMalloc_8u: function (len: int): PIpp8u; extdecl;
  ippsFree: procedure (ptr: Pointer); extdecl;
  ippsSubCRev_8u_ISfs: function (val: Ipp8u; pSrcDst: PIpp8u; len: int; scaleFactor: int): IppStatus; extdecl;
  ippsSubCRev_8u_Sfs: function(pSrc: PIpp8u; val: Ipp8u; pDst: PIpp8u; len: int; scaleFactor: int): IppStatus; extdecl;

  // Image Processing (ippi.h)
  ippiGetLibVersion: function: PIppLibraryVersion; extdecl;
  ippiMalloc_8u_C1 : function(widthPixels, heightPixels: int; var StepBytes: int): PIpp8u; extdecl;
  ippiMalloc_16u_C1: function(widthPixels, heightPixels: int; var StepBytes: int): PIpp16u; extdecl;
  ippiMalloc_16s_C1: function(widthPixels, heightPixels: int; var StepBytes: int): PIpp16s; extdecl;
  ippiMalloc_32f_C1: function(widthPixels, heightPixels: int; var StepBytes: int): PIpp32f; extdecl;
  ippiFree: procedure (ptr: Pointer); extdecl;
  ippiAdd_8u_C1IRSfs: function(pSrc: PIpp8u; srcStep: int; pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; scaleFactor: int): IppStatus; extdecl;
  ippiSub_8u_C1IRSfs: function(pSrc: PIpp8u; srcStep: int; pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; scaleFactor: int): IppStatus; extdecl;
  ippiSub_8u_C1RSfs: function(pSrc1: PIpp8u; src1Step: int; pSrc2: PIpp8u; src2Step: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; scaleFactor: int): IppStatus; extdecl;
  ippiXorC_8u_C1IR: function(value: Ipp8u; pSrcDst: PIpp8u; srcDstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiXorC_8u_C1R: function(pSrc: PIpp8u; srcStep: int; value: Ipp8u; pDst: PIpp8u; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiXorC_16u_C1IR: function(value: Ipp16u; pSrcDst: PIpp16u; srcDstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiCopy_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize): IppStatus; extdecl;
  ippiCopy_32f_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int;
      roiSize: IppiSize): IppStatus; extdecl;
  ippiSet_8u_C1R: function(value: Ipp8u; pDst: PIpp8u; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiSet_16u_C1R: function(value: Ipp16u; pDst: PIpp16u; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiCopyConstBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize; pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize;
      topBorderHeight, leftBorderWidth: int; value: Ipp8u): IppStatus; extdecl;
  ippiCopyConstBorder_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int; srcRoiSize: IppiSize;
      dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int; value: Ipp8u): IppStatus; extdecl;
  ippiCopyReplicateBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize;
      pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int): IppStatus; extdecl;
  ippiCopyReplicateBorder_8u_C1IR: function(pSrc: PIpp8u; srcStep: int; srcRoiSize: IppiSize;
      dstRoiSize: IppiSize; topBorderHeight, leftBorderWidth: int): IppStatus; extdecl;
  ippiScaleC_8u_C1IR: function (pSrcDst: PIpp8u; srcDstStep: int; mVal, aVal: Ipp64f;
      roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus; extdecl;
  ippiScaleC_8u_C1R: function (pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus; extdecl;
  ippiScaleC_8u16s_C1R: function (pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp16s; dstStep: int;
      roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus; extdecl;
  ippiScaleC_8u32f_C1R: function (pSrc: PIpp8u; srcStep: int; mVal, aVal: Ipp64f; pDst: PIpp32f; dstStep: int;
      roiSize: IppiSize; hint: IppHintAlgorithm): IppStatus; extdecl;
  ippiCompareC_8u_C1R: function (pSrc: PIpp8u; srcStep: int; value: Ipp8u;
      pDst: PIpp8u; dstStep: int; roiSize: IppiSize; ippCmpOp: IppCmpOp): IppStatus; extdecl;
  ippiConvert_8u16u_C1R: function(pSrc: PIpp8u;  srcStep: int; pDst: PIpp16u; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_8u16s_C1R: function(pSrc: PIpp8u;  srcStep: int; pDst: PIpp16s; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_8u32s_C1R: function(pSrc: PIpp8u;  srcStep: int; pDst: PIpp32s; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_8u32f_C1R: function(pSrc: PIpp8u;  srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_16u32s_C1R: function(pSrc: PIpp16u; srcStep: int; pDst: PIpp32s; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_16s8u_C1R: function(pSrc: PIpp16s; srcStep: int; pDst: PIpp8u;  dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiConvert_32f8u_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp8u;  dstStep: int; roiSize: IppiSize; roundMode: IppRoundMode): IppStatus; extdecl;
  ippiConvert_32f32s_C1RSfs: function (pSrc: PIpp32f; srcStep: int; pDst: PIpp32s; dstStep: int; roi: IppiSize; round: IppRoundMode; scaleFactor: int): IppStatus; extdecl;

  ippiHistogramGetBufferSize: function(dataType: IppDataType; roiSize: IppiSize; nLevels: Pint; numChannels: int; uniform: int; var SpecSize: int; var BufferSize: int): IppStatus; extdecl;
  ippiHistogramUniformInit: function(dataType: IppDataType; lowerLevel, upperLevel: PIpp32f; nLevels: Pint; numChannels: int; pSpec: PIppiHistogramSpec): IppStatus; extdecl;
  ippiHistogram_8u_C1R: function(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; pHist: PIpp32u; pSpec: PIppiHistogramSpec; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiThreshold_GT_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; threshold: Ipp8u): IppStatus; extdecl;
  ippiThreshold_LTVal_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; threshold: Ipp8u; value: Ipp8u): IppStatus; extdecl;
  ippiThreshold_LTVal_16u_C1IR: function(pSrcDst: PIpp16u; srcDstStep: int;
      roiSize: IppiSize; threshold: Ipp16u; value: Ipp16u): IppStatus; extdecl;
  ippiThreshold_LTVal_32f_C1IR: function(pSrcDst: PIpp32f; srcDstStep: int ;
      roiSize: IppiSize; threshold: Ipp32f; value: Ipp32f): IppStatus; extdecl;
  ippiThreshold_GTVal_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; threshold: Ipp8u; value: Ipp8u): IppStatus; extdecl;
  ippiThreshold_GTVal_16u_C1IR: function(pSrcDst: PIpp16u; srcDstStep: int;
      roiSize: IppiSize; threshold: Ipp16u; value: Ipp16u): IppStatus; extdecl;
  ippiThreshold_LTValGTVal_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int;
      roiSize: IppiSize; thresholdLT: Ipp8u; valueLT: Ipp8u; thresholdGT: Ipp8u; valueGT: Ipp8u): IppStatus; extdecl;
  ippiComputeThreshold_Otsu_8u_C1R: function(pSrc: PIpp8u; srcStep: int;
      roiSize: IppiSize; var Threshold: Ipp8u): IppStatus; extdecl;
  ippiThresholdAdaptiveBoxGetBufferSize: function(roiSize: IppiSize; maskSize: IppiSize; dataType: IppDataType;
      numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiThresholdAdaptiveBox_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize; maskSize: IppiSize;
      delta: Ipp32f; valGT, valLE: Ipp8u; borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiThresholdAdaptiveBox_8u_C1IR: function(pSrcDst: PIpp8u; srcDstStep: int; roiSize: IppiSize; maskSize: IppiSize;
      delta: Ipp32f; valGT, valLE: Ipp8u; borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiResizeGetSize_8u: function(srcSize: IppiSize; dstSize: IppiSize; interpolation: IppiInterpolationType;
      antialiasing: Ipp32u; var SpecSize, InitBufSize: int): IppStatus; extdecl;
  ippiResizeGetBufferSize_8u: function(pSpec: PIppiResizeSpec_32f; dstSize: IppiSize; numChannels: Ipp32u; var BufSize: int): IppStatus; extdecl;
  ippiResizeNearestInit_8u: function(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus; extdecl;
  ippiResizeLinearInit_8u: function(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus; extdecl;
  ippiResizeSuperInit_8u: function(srcSize: IppiSize; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f): IppStatus; extdecl;
  ippiResizeNearest_8u_C1R: function(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
      dstOffset: IppiPoint; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiResizeLinear_8u_C1R: function(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
      dstOffset: IppiPoint; dstSize: IppiSize; border: IppiBorderType; pBorderValue: PIpp8u;
      pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiResizeSuper_8u_C1R: function(pSrc: PIpp8u; srcStep: Ipp32s; pDst: PIpp8u; dstStep: Ipp32s;
      dstOffset: IppiPoint; dstSize: IppiSize; pSpec: PIppiResizeSpec_32f; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiGetAffineBound: function(srcROI: IppiRect; var bound: array_double_2_2; const coeffs: array_double_2_3): IppStatus; extdecl;
  ippiGetAffineQuad: function(srcROI: IppiRect; var quad: array_double_4_2; const coeffs: array_double_2_3): IppStatus; extdecl;
  ippiGetRotateShift: function(xCenter, yCenter: double; angle: double; var xShift, yShift: double): IppStatus; extdecl;
  ippiGetRotateTransform: function(angle: double; xShift, yShift: double; var coeffs: array_double_2_3): IppStatus; extdecl;
  ippiWarpGetBufferSize: function(pSpec: PIppiWarpSpec; dstRoiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiWarpAffineGetSize: function(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
      coeffs: array_double_2_3; interpolation: IppiInterpolationType; direction: IppiWarpDirection;
      borderType: IppiBorderType; var SpecSize: int; var InitBufSize: int): IppStatus; extdecl;
  ippiWarpAffineNearestInit: function(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
      coeffs: array_double_2_3; direction: IppiWarpDirection; numChannels: int; borderType: IppiBorderType;
      pBorderValue: PIpp64f; smoothEdge: int; pSpec: PIppiWarpSpec): IppStatus; extdecl;
  ippiWarpAffineLinearInit: function(srcSize: IppiSize; dstSize: IppiSize; dataType: IppDataType;
      coeffs: array_double_2_3; direction: IppiWarpDirection; numChannels: int; borderType: IppiBorderType;
      pBorderValue: PIpp64f; smoothEdge: int; pSpec: PIppiWarpSpec): IppStatus; extdecl;
  ippiWarpAffineNearest_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      dstRoiOffset: IppiPoint; dstRoiSize: IppiSize; pSpec: PIppiWarpSpec; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiWarpAffineLinear_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      dstRoiOffset: IppiPoint; dstRoiSize: IppiSize; pSpec: PIppiWarpSpec; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiFFTGetSize_R_32f: function(orderX, orderY: int; flag: int; hint: IppHintAlgorithm; var pSizeSpec, pSizeInit, pSizeBuf: int): IppStatus; extdecl;
  ippiDFTGetSize_R_32f: function(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; var pSizeSpec, pSizeInit, pSizeBuf: int): IppStatus; extdecl;
  ippiDFTGetSize_C_32fc: function(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; var pSizeSpec, pSizeInit, pSizeBuf: int): IppStatus; extdecl;
  ippiFFTInit_R_32f: function(orderX, orderY: int; flag: int; hint: IppHintAlgorithm; pFFTSpec: PIppiFFTSpec_R_32f; pMemInit: PIpp8u): IppStatus; extdecl;
  ippiDFTInit_R_32f: function(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; pDFTSpec: PIppiDFTSpec_R_32f; pMemInit: PIpp8u): IppStatus; extdecl;
  ippiDFTInit_C_32fc: function(roiSize: IppiSize; flag: int; hint: IppHintAlgorithm; pDFTSpec: PIppiDFTSpec_C_32fc; pMemInit: PIpp8u): IppStatus; extdecl;
  ippiFFTFwd_RToPack_32f_C1IR: function(pSrcDst: PIpp32f; srcDstStep: int; pFFTSpec: PIppiFFTSpec_R_32f; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiDFTFwd_RToPack_32f_C1IR: function(pSrcDst: PIpp32f; srcDstStep: int; pDFTSpec: PIppiDFTSpec_R_32f; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiDFTFwd_CToC_32fc_C1IR: function(pSrcDst: PIpp32fc; srcDstStep: int; pDFTSpec: PIppiDFTSpec_C_32fc; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiPhasePackGetBufferSize_32f: function(numChannels: int; dstRoiSize: IppiSize; var pSize: int): IppStatus; extdecl;
  ippiMagnitudePackGetBufferSize_32f: function(numChannels: int; dstRoiSize: IppiSize; var pSize: int): IppStatus; extdecl;
  ippiPhasePack_32f_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; dstRoiSize: IppiSize; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMagnitudePack_32f_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; dstRoiSize: IppiSize; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiPhase_32fc32f_C1R: function(pSrc: PIpp32fc; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;
  ippiMagnitude_32fc32f_C1R: function(pSrc: PIpp32fc; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize): IppStatus; extdecl;

  ippiMomentGetStateSize_64f: function(hint: IppHintAlgorithm; var pSize: int): IppStatus; extdecl;
  ippiMomentInit_64f: function(pState: PIppiMomentState_64f; hint: IppHintAlgorithm): IppStatus; extdecl;
  ippiMoments64f_8u_C1R: function(pSrc: PIpp8u; srcStep:int; roiSize: IppiSize; pCtx: PIppiMomentState_64f): IppStatus; extdecl;
  ippiGetSpatialMoment_64f: function(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel:int; roiOffset: IppiPoint; var pValue: Ipp64f): IppStatus; extdecl;
  ippiGetCentralMoment_64f: function(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel: int; var pValue: Ipp64f): IppStatus; extdecl;
  ippiGetNormalizedCentralMoment_64f: function(pState: PIppiMomentState_64f; mOrd, nOrd: int; nChannel: int; var pValue: Ipp64f): IppStatus; extdecl;
  ippiGetHuMoments_64f: function(pState: PIppiMomentState_64f; nChannel: int; pHm: PIppiHuMoment_64f): IppStatus; extdecl;

  ippiFilterBorderGetSize: function(kernelSize: IppiSize; dstRoiSize: IppiSize; dataType: IppDataType;
      kernelType: IppDataType; numChannels: int; var SpecSize: int; var BufferSize:int): IppStatus; extdecl;
  ippiFilterBorderInit_16s: function(pKernel: PIpp16s; kernelSize: IppiSize; divisor: int; dataType: IppDataType;
      numChannels: int; roundMode: IppRoundMode; pSpec: PIppiFilterBorderSpec): IppStatus; extdecl;
  ippiFilterBorderInit_32f: function(pKernel: PIpp32f; kernelSize: IppiSize; dataType: IppDataType;
      numChannels: int; roundMode: IppRoundMode; pSpec: PIppiFilterBorderSpec): IppStatus; extdecl;
  ippiFilterBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      dstRoiSize: IppiSize; border: IppiBorderType; borderValue: PIpp8u;
      pSpec: PIppiFilterBorderSpec; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterBorder_32f_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int;
      dstRoiSize: IppiSize; border: IppiBorderType; borderValue: PIpp32f;
      pSpec: PIppiFilterBorderSpec; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterMedianBorderGetBufferSize: function(dstRoiSize: IppiSize; maskSize: IppiSize;
      dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterMedianBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int;
      pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; maskSize: IppiSize;
      borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiFilterPrewittHorizBorderGetBufferSize: function(dstRoiSize: IppiSize; mask: IppiMaskSize;
      srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterPrewittVertBorderGetBufferSize: function (dstRoiSize: IppiSize; mask: IppiMaskSize;
      srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterLaplaceBorderGetBufferSize: function(dstRoiSize: IppiSize; mask: IppiMaskSize;
      srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterSobelGetBufferSize: function(dstRoiSize: IppiSize; mask: IppiMaskSize; normType: IppNormType;
      srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterSharpenBorderGetBufferSize: function(dstRoiSize: IppiSize; mask: IppiMaskSize;
      srcDataType, dstDataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterPrewittVertBorder_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
      borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterPrewittHorizBorder_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
      borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterLaplaceBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      dstRoiSize: IppiSize; mask: IppiMaskSize; borderType: IppiBorderType; borderValue: Ipp8u;
      pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterSobel_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp16s; dstStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType; borderType: IppiBorderType;
      borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFilterSharpenBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; borderType: IppiBorderType;
      borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiGradientVectorGetBufferSize: function(roiSize: IppiSize; mask: IppiMaskSize;
      dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiGradientVectorPrewitt_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int;
      pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
      pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
      borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiGradientVectorSobel_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int;
      pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
      pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
      borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiGradientVectorScharr_8u16s_C1R: function(pSrc: PIpp8u; srcStep: int;
      pGx: PIpp16s; gxStep: int; pGy: PIpp16s; gyStep: int;
      pMag: PIpp16s; magStep: int; pAngle: PIpp32f; angleStep: int;
      dstRoiSize: IppiSize; maskSize: IppiMaskSize; normType: IppNormType;
      borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiHOGGetSize: function(pConfig: PIppiHOGConfig; var pHOGSpecSize: int): IppStatus; extdecl;
  ippiHOGInit: function(pConfig: PIppiHOGConfig; pHOGSpec: PIppiHOGSpec): IppStatus; extdecl;
  ippiHOGGetBufferSize: function(pHOGSpec: PIppiHOGSpec; roiSize: IppiSize; var pBufferSize: int): IppStatus; extdecl;
  ippiHOGGetDescriptorSize: function(pHOGSpec: PIppiHOGSpec; var pWinDescriptorSize: int): IppStatus; extdecl;
  ippiHOG_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; pLocation: PIppiPoint; nLocations: int; pDst: PIpp32f; pHOGSpec: PIppiHOGSpec; borderID: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiLBPImageMode3x3_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; mode: int; borderType: IppiBorderType; borderValue: PIpp8u): IppStatus; extdecl;

  ippiFilterWienerGetBufferSize: function(dstRoiSize: IppiSize; maskSize: IppiSize; channels: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterWiener_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; dstRoiSize: IppiSize; maskSize: IppiSize; anchor: IppiPoint; noise: PIpp32f; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiDeconvLRGetSize_32f: function (numChannels, kernelSize: int; maxRoi: IppiSize; var Size: int): IppStatus; extdecl;
  ippiDeconvLRInit_32f_C1R: function (pDeconvLR: PIppiDeconvLR_32f_C1R; pKernel: PIpp32f; kernelSize: int; maxRoi: IppiSize; threshold: Ipp32f): IppStatus; extdecl;
  ippiDeconvLR_32f_C1R: function(pSrc: PIpp32f; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize; numIter: int; pDeconvLR: PIppiDeconvLR_32f_C1R): IppStatus; extdecl;

  // Computer Vision (ippcv.h)
  ippiSegmentWatershedGetBufferSize_8u16u_C1R : function(roiSize: IppiSize; var pBufSize: int): IppStatus; extdecl;
  ippiSegmentWatershed_8u16u_C1IR: function(pSrc: PIpp8u; srcStep: int; pMarker: PIpp16u; markerStep: int; roiSize: IppiSize; norm: IppiNorm; flag: int; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiBoundSegments_16u_C1IR: function(pMarker: PIpp16u; markerStep: int; roiSize: IppiSize; val: Ipp16u; norm: IppiNorm): IppStatus; extdecl;

  ippiLabelMarkersGetBufferSize_8u32s_C1R: function(roiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiLabelMarkersGetBufferSize_16u_C1R: function(roiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiLabelMarkers_8u32s_C1R: function(pSrcMarker: PIpp8u; srcMarkerStep: int; pDstMarker: PIpp32s; dstMarkerStep: int;
      roiSize: IppiSize; minLabel, maxLabel: int; norm: IppiNorm; var Number: int; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiLabelMarkers_16u_C1IR: function(pMarker: PIpp16u; markerStep: int; roiSize: IppiSize;
      minLabel, maxLabel: int; norm: IppiNorm; var Number: int; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiFilterGaussianGetBufferSize: function(maxRoiSize: IppiSize; kernelSize: Ipp32u;
      dataType: IppDataType; numChannels: int; var SpecSize: int; var BufferSize: int): IppStatus; extdecl;
  ippiFilterGaussianInit: function(roiSize: IppiSize; kernelSize: Ipp32u; sigma: Ipp32f;
      borderType: IppiBorderType; dataType: IppDataType; numChannels: int;
      Spec: PIppFilterGaussianSpec; Buffer: PIpp8u): IppStatus; extdecl;
  ippiFilterGaussianBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int;
      pDst: PIpp8u; dstStep: int; roiSize: IppiSize; borderValue: Ipp8u;
      Spec: PIppFilterGaussianSpec; Buffer: PIpp8u): IppStatus; extdecl;

  ippiDistanceTransform_3x3_8u16u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp16u; dstStep:int; roiSize: IppiSize; pMetrics: PIpp32s): IppStatus; extdecl;
  ippiDistanceTransform_3x3_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep:int; roiSize: IppiSize; pMetrics: PIpp32f): IppStatus; extdecl;
  ippiTrueDistanceTransformGetBufferSize_8u32f_C1R: function(roiSize: IppiSize; var BufferSize: int): IppStatus; extdecl;
  ippiTrueDistanceTransform_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep:int; roiSize: IppiSize; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiMorphologyBorderGetSize_8u_C1R: function(roiSize: IppiSize; maskSize: IppiSize; var SpecSize: int; var BufferSize: int): IppStatus; extdecl;
  ippiMorphologyBorderInit_8u_C1R: function(roiSize: IppiSize; pMask: PIpp8u; maskSize: IppiSize;
      pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiDilateBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize;
      borderType: IppiBorderType; borderValue: Ipp8u; pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiErodeBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int; roiSize: IppiSize;
      borderType: IppiBorderType; borderValue: Ipp8u; pMorphSpec: PIppiMorphState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphAdvGetSize_8u_C1R: function(roiSize: IppiSize; maskSize: IppiSize; var SpecSize: int; var BufferSize: int): IppStatus; extdecl;
  ippiMorphAdvInit_8u_C1R: function(roiSize: IppiSize; pMask: PIpp8u; maskSize: IppiSize;
      pMorphSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphOpenBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphCloseBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphTophatBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphBlackhatBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiMorphGradientBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; borderType: IppiBorderType; borderValue: Ipp8u; pMorthSpec: PIppiMorphAdvState; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiHoughLineGetSize_8u_C1R: function(roiSize: IppiSize; delta: IppPointPolar; maxLineCount: int; var BufSize: int): IppStatus; extdecl;
  ippiHoughLine_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize; delta: IppPointPolar; threshold: int;
      pLine: PIppPointPolar; maxLineCount: int; var LineCount: int; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiHoughLine_Region_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; roiSize: IppiSize;
      pLine: PIppPointPolar; dstRoi: PIppPointPolar; maxLineCount: int; var LineCount: int; delta: IppPointPolar; threshold: int; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiCannyBorderGetSize: function(roiSize: IppiSize; filterType: IppiDifferentialKernel;
      mask: IppiMaskSize; dataType: IppDataType; var BufferSize: int ): IppStatus; extdecl;
  ippiCannyBorder_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp8u; dstStep: int;
      roiSize: IppiSize; filterType: IppiDifferentialKernel; mask: IppiMaskSize;
      borderType: IppiBorderType; borderValue: Ipp8u;
      lowThresh, highThresh: Ipp32f; norm: IppNormType; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiHarrisCornerGetBufferSize: function(roiSize: IppiSize; filterMask: IppiMaskSize;
      avgWndSize: Ipp32u; dataType: IppDataType; numChannels: int; var BufferSize: int): IppStatus; extdecl;
  ippiHarrisCorner_8u32f_C1R: function(pSrc: PIpp8u; srcStep: int; pDst: PIpp32f; dstStep: int; roiSize: IppiSize;
      filterType: IppiDifferentialKernel; filterMask: IppiMaskSize; avgWndSize: Ipp32u; k, scale: float;
      borderType: IppiBorderType; borderValue: Ipp8u; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiFastNGetSize: function(srcSize: IppiSize; circleRadius, N, orientationBins, option: int;
      dataType: IppDataType; nChannels: int; var pSpecSize: int): IppStatus; extdecl;
  ippiFastNInit: function(srcSize: IppiSize; circleRadius, N, orientationBins, option: int;
      threshold: Ipp32f; dataType: IppDataType; nChannels: int; pSpec: PIppiFastNSpec): IppStatus; extdecl;
  ippiFastNGetBufferSize: function(pSpec: PIppiFastNSpec; dstRoiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiFastN_8u_C1R: function(pSrc: PIpp8u; srcStep: int; pDstCorner: PIpp8u; dstCornerStep: int;
      pDstScore: PIpp8u; dstScoreStep: int; pNumCorner: pint; srcRoiOffset: IppiPoint; dstRoiSize: IppiSize;
      pSpec: PIppiFastNSpec; pBuffer: PIpp8u): IppStatus; extdecl;

  ippiFloodFillGetSize: function(roiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiFloodFillGetSize_Grad: function(roiSize: IppiSize; var BufSize: int): IppStatus; extdecl;
  ippiFloodFill_4Con_8u_C1IR: function(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
      seed: IppiPoint; newVal: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFloodFill_8Con_8u_C1IR: function(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
      seed: IppiPoint; newVal: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFloodFill_Range4Con_8u_C1IR: function(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
      seed: IppiPoint; newVal: Ipp8u; minDelta, maxDelta: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFloodFill_Range8Con_8u_C1IR: function(pImage: PIpp8u; imageStep: int; roiSize: IppiSize;
      seed: IppiPoint; newVal: Ipp8u; minDelta, maxDelta: Ipp8u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus; extdecl;
  ippiFloodFill_Range8Con_16u_C1IR: function(pImage: PIpp16u; imageStep: int; roiSize: IppiSize;
      seed: IppiPoint; newVal: Ipp16u; minDelta, maxDelta: Ipp16u; pRegion: PIppiConnectedComp; pBuffer: PIpp8u): IppStatus; extdecl;
{$ENDIF}

procedure LoadIPP(APath: string = '');
procedure UnloadIPP;

function ROI(width, height: integer): IppiSize; inline;
function POI(x,y: integer): IppiPoint; inline;

implementation

{$IFDEF LOAD_DYNAMICALLY}
{$IFOPT D+} // DEBUGINFO ON
uses
  SysUtils;
{$ENDIF}

var
  IppCoreLibHandle,
  IppsLibHandle,
  IppiLibHandle,
  IppcvLibHandle: TLibHandle;
  RefCount: integer;

procedure LoadIPP(APath: string);
  function LoadIppLib(const AFileName: string): TLibHandle;
  begin
    Result := System.LoadLibrary(AFileName);
    if Result = NilHandle then begin
      RefCount := 0;
{$IF DEFINED(EInOutError)}
      raise EInOutError.CreateFmt('Can not load library "%s". Check your installation.'+LineEnding+'%s',
                                  [AFileName, SysErrorMessage(GetLastOSError)]);
{$ENDIF}
    end;
  end;
begin
  Inc(RefCount);
  if RefCount = 1 then begin
    if APath <> '' then
{$IFOPT D+}
      if SysUtils.DirectoryExists(APath) then
        APath := SysUtils.IncludeTrailingPathDelimiter(APath)
      else
        APath := '';
{$ELSE}
      if APath[Length(APath)] <> System.DirectorySeparator then
        APath := APath + System.DirectorySeparator;
{$ENDIF}
    IppCoreLibHandle := LoadIppLib(APath+libippcore);
    IppsLibHandle    := LoadIppLib(APath+libipps);
    IppiLibHandle    := LoadIppLib(APath+libippi);
    IppcvLibHandle   := LoadIppLib(APath+libippcv);

    pointer(ippInit) := GetProcedureAddress(IppCoreLibHandle,'ippInit');
    pointer(ippGetLibVersion) := GetProcedureAddress(IppCoreLibHandle,'ippGetLibVersion');
    pointer(ippSetCpuFeatures) := GetProcedureAddress(IppCoreLibHandle,'ippSetCpuFeatures');

    // Signal Processing (ipps.h)
    pointer(ippsMalloc_8u) := GetProcedureAddress(IppsLibHandle,'ippsMalloc_8u');
    pointer(ippsFree) := GetProcedureAddress(IppsLibHandle,'ippsFree');
    pointer(ippsSubCRev_8u_ISfs) := GetProcedureAddress(IppsLibHandle,'ippsSubCRev_8u_ISfs');
    pointer(ippsSubCRev_8u_Sfs) := GetProcedureAddress(IppsLibHandle,'ippsSubCRev_8u_Sfs');

    // Image Processing (ippi.h)
    pointer(ippiGetLibVersion) := GetProcedureAddress(IppiLibHandle,'ippiGetLibVersion');
    pointer(ippiMalloc_8u_C1) := GetProcedureAddress(IppiLibHandle,'ippiMalloc_8u_C1');
    pointer(ippiMalloc_16u_C1) := GetProcedureAddress(IppiLibHandle,'ippiMalloc_16u_C1');
    pointer(ippiMalloc_16s_C1) := GetProcedureAddress(IppiLibHandle,'ippiMalloc_16s_C1');
    pointer(ippiMalloc_32f_C1) := GetProcedureAddress(IppiLibHandle,'ippiMalloc_32f_C1');
    pointer(ippiFree) := GetProcedureAddress(IppiLibHandle,'ippiFree');
    pointer(ippiAdd_8u_C1IRSfs) := GetProcedureAddress(IppiLibHandle,'ippiAdd_8u_C1IRSfs');
    pointer(ippiSub_8u_C1IRSfs) := GetProcedureAddress(IppiLibHandle,'ippiSub_8u_C1IRSfs');
    pointer(ippiSub_8u_C1RSfs) := GetProcedureAddress(IppiLibHandle,'ippiSub_8u_C1RSfs');
    pointer(ippiXorC_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiXorC_8u_C1IR');
    pointer(ippiXorC_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiXorC_8u_C1R');
    pointer(ippiXorC_16u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiXorC_16u_C1IR');
    pointer(ippiCopy_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiCopy_8u_C1R');
    pointer(ippiCopy_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiCopy_32f_C1R');
    pointer(ippiSet_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiSet_8u_C1R');
    pointer(ippiSet_16u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiSet_16u_C1R');
    pointer(ippiCopyConstBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiCopyConstBorder_8u_C1R');
    pointer(ippiCopyConstBorder_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiCopyConstBorder_8u_C1IR');
    pointer(ippiCopyReplicateBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiCopyReplicateBorder_8u_C1R');
    pointer(ippiCopyReplicateBorder_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiCopyReplicateBorder_8u_C1IR');
    pointer(ippiScaleC_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiScaleC_8u_C1IR');
    pointer(ippiScaleC_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiScaleC_8u_C1R');
    pointer(ippiScaleC_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiScaleC_8u16s_C1R');
    pointer(ippiScaleC_8u32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiScaleC_8u32f_C1R');
    pointer(ippiCompareC_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiCompareC_8u_C1R');
    pointer(ippiConvert_8u16u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_8u16u_C1R');
    pointer(ippiConvert_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_8u16s_C1R');
    pointer(ippiConvert_8u32s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_8u32s_C1R');
    pointer(ippiConvert_8u32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_8u32f_C1R');
    pointer(ippiConvert_16u32s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_16u32s_C1R');
    pointer(ippiConvert_16s8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_16s8u_C1R');
    pointer(ippiConvert_32f8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiConvert_32f8u_C1R');
    pointer(ippiConvert_32f32s_C1RSfs) := GetProcedureAddress(IppiLibHandle,'ippiConvert_32f32s_C1RSfs');

    pointer(ippiHistogramGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiHistogramGetBufferSize');
    pointer(ippiHistogramUniformInit) := GetProcedureAddress(IppiLibHandle,'ippiHistogramUniformInit');
    pointer(ippiHistogram_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiHistogram_8u_C1R');

    pointer(ippiThreshold_GT_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_GT_8u_C1IR');
    pointer(ippiThreshold_LTVal_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_LTVal_8u_C1IR');
    pointer(ippiThreshold_LTVal_16u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_LTVal_16u_C1IR');
    pointer(ippiThreshold_LTVal_32f_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_LTVal_32f_C1IR');
    pointer(ippiThreshold_GTVal_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_GTVal_8u_C1IR');
    pointer(ippiThreshold_GTVal_16u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_GTVal_16u_C1IR');
    pointer(ippiThreshold_LTValGTVal_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThreshold_LTValGTVal_8u_C1IR');
    pointer(ippiComputeThreshold_Otsu_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiComputeThreshold_Otsu_8u_C1R');
    pointer(ippiThresholdAdaptiveBoxGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiThresholdAdaptiveBoxGetBufferSize');
    pointer(ippiThresholdAdaptiveBox_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiThresholdAdaptiveBox_8u_C1R');
    pointer(ippiThresholdAdaptiveBox_8u_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiThresholdAdaptiveBox_8u_C1IR');

    pointer(ippiResizeGetSize_8u) := GetProcedureAddress(IppiLibHandle,'ippiResizeGetSize_8u');
    pointer(ippiResizeGetBufferSize_8u) := GetProcedureAddress(IppiLibHandle,'ippiResizeGetBufferSize_8u');
    pointer(ippiResizeNearestInit_8u) := GetProcedureAddress(IppiLibHandle,'ippiResizeNearestInit_8u');
    pointer(ippiResizeLinearInit_8u) := GetProcedureAddress(IppiLibHandle,'ippiResizeLinearInit_8u');
    pointer(ippiResizeSuperInit_8u) := GetProcedureAddress(IppiLibHandle,'ippiResizeSuperInit_8u');
    pointer(ippiResizeNearest_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiResizeNearest_8u_C1R');
    pointer(ippiResizeLinear_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiResizeLinear_8u_C1R');
    pointer(ippiResizeSuper_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiResizeSuper_8u_C1R');

    pointer(ippiGetAffineBound) := GetProcedureAddress(IppiLibHandle,'ippiGetAffineBound');
    pointer(ippiGetAffineQuad) := GetProcedureAddress(IppiLibHandle,'ippiGetAffineQuad');
    pointer(ippiGetRotateShift) := GetProcedureAddress(IppiLibHandle,'ippiGetRotateShift');
    pointer(ippiGetRotateTransform) := GetProcedureAddress(IppiLibHandle,'ippiGetRotateTransform');
    pointer(ippiWarpGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiWarpGetBufferSize');
    pointer(ippiWarpAffineGetSize) := GetProcedureAddress(IppiLibHandle,'ippiWarpAffineGetSize');
    pointer(ippiWarpAffineNearestInit) := GetProcedureAddress(IppiLibHandle,'ippiWarpAffineNearestInit');
    pointer(ippiWarpAffineLinearInit) := GetProcedureAddress(IppiLibHandle,'ippiWarpAffineLinearInit');
    pointer(ippiWarpAffineNearest_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiWarpAffineNearest_8u_C1R');
    pointer(ippiWarpAffineLinear_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiWarpAffineLinear_8u_C1R');

    pointer(ippiFFTGetSize_R_32f) := GetProcedureAddress(IppiLibHandle,'ippiFFTGetSize_R_32f');
    pointer(ippiDFTGetSize_R_32f) := GetProcedureAddress(IppiLibHandle,'ippiDFTGetSize_R_32f');
    pointer(ippiDFTGetSize_C_32fc) := GetProcedureAddress(IppiLibHandle,'ippiDFTGetSize_C_32fc');
    pointer(ippiFFTInit_R_32f) := GetProcedureAddress(IppiLibHandle,'ippiFFTInit_R_32f');
    pointer(ippiDFTInit_R_32f) := GetProcedureAddress(IppiLibHandle,'ippiDFTInit_R_32f');
    pointer(ippiDFTInit_C_32fc) := GetProcedureAddress(IppiLibHandle,'ippiDFTInit_C_32fc');
    pointer(ippiFFTFwd_RToPack_32f_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiFFTFwd_RToPack_32f_C1IR');
    pointer(ippiDFTFwd_RToPack_32f_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiDFTFwd_RToPack_32f_C1IR');
    pointer(ippiDFTFwd_CToC_32fc_C1IR) := GetProcedureAddress(IppiLibHandle,'ippiDFTFwd_CToC_32fc_C1IR');
    pointer(ippiPhasePackGetBufferSize_32f) := GetProcedureAddress(IppiLibHandle,'ippiPhasePackGetBufferSize_32f');
    pointer(ippiMagnitudePackGetBufferSize_32f) := GetProcedureAddress(IppiLibHandle,'ippiMagnitudePackGetBufferSize_32f');
    pointer(ippiPhasePack_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiPhasePack_32f_C1R');
    pointer(ippiMagnitudePack_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiMagnitudePack_32f_C1R');
    pointer(ippiPhase_32fc32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiPhase_32fc32f_C1R');
    pointer(ippiMagnitude_32fc32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiMagnitude_32fc32f_C1R');

    pointer(ippiMomentGetStateSize_64f) := GetProcedureAddress(IppiLibHandle,'ippiMomentGetStateSize_64f');
    pointer(ippiMomentInit_64f) := GetProcedureAddress(IppiLibHandle,'ippiMomentInit_64f');
    pointer(ippiMoments64f_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiMoments64f_8u_C1R');
    pointer(ippiGetSpatialMoment_64f) := GetProcedureAddress(IppiLibHandle,'ippiGetSpatialMoment_64f');
    pointer(ippiGetCentralMoment_64f) := GetProcedureAddress(IppiLibHandle,'ippiGetCentralMoment_64f');
    pointer(ippiGetNormalizedCentralMoment_64f) := GetProcedureAddress(IppiLibHandle,'ippiGetNormalizedCentralMoment_64f');
    pointer(ippiGetHuMoments_64f) := GetProcedureAddress(IppiLibHandle,'ippiGetHuMoments_64f');

    pointer(ippiFilterBorderGetSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterBorderGetSize');
    pointer(ippiFilterBorderInit_16s) := GetProcedureAddress(IppiLibHandle,'ippiFilterBorderInit_16s');
    pointer(ippiFilterBorderInit_32f) := GetProcedureAddress(IppiLibHandle,'ippiFilterBorderInit_32f');
    pointer(ippiFilterBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterBorder_8u_C1R');
    pointer(ippiFilterBorder_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterBorder_32f_C1R');
    pointer(ippiFilterMedianBorderGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterMedianBorderGetBufferSize');
    pointer(ippiFilterMedianBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterMedianBorder_8u_C1R');
    pointer(ippiFilterPrewittHorizBorderGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterPrewittHorizBorderGetBufferSize');
    pointer(ippiFilterPrewittVertBorderGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterPrewittVertBorderGetBufferSize');
    pointer(ippiFilterLaplaceBorderGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterLaplaceBorderGetBufferSize');
    pointer(ippiFilterSobelGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterSobelGetBufferSize');
    pointer(ippiFilterSharpenBorderGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterSharpenBorderGetBufferSize');
    pointer(ippiFilterPrewittVertBorder_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterPrewittVertBorder_8u16s_C1R');
    pointer(ippiFilterPrewittHorizBorder_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterPrewittHorizBorder_8u16s_C1R');
    pointer(ippiFilterLaplaceBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterLaplaceBorder_8u_C1R');
    pointer(ippiFilterSobel_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterSobel_8u16s_C1R');
    pointer(ippiFilterSharpenBorder_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterSharpenBorder_8u_C1R');

    pointer(ippiGradientVectorGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiGradientVectorGetBufferSize');
    pointer(ippiGradientVectorPrewitt_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiGradientVectorPrewitt_8u16s_C1R');
    pointer(ippiGradientVectorSobel_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiGradientVectorSobel_8u16s_C1R');
    pointer(ippiGradientVectorScharr_8u16s_C1R) := GetProcedureAddress(IppiLibHandle,'ippiGradientVectorScharr_8u16s_C1R');

    pointer(ippiHOGGetSize) := GetProcedureAddress(IppiLibHandle,'ippiHOGGetSize');
    pointer(ippiHOGInit) := GetProcedureAddress(IppiLibHandle,'ippiHOGInit');
    pointer(ippiHOGGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiHOGGetBufferSize');
    pointer(ippiHOGGetDescriptorSize) := GetProcedureAddress(IppiLibHandle,'ippiHOGGetDescriptorSize');
    pointer(ippiHOG_8u32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiHOG_8u32f_C1R');
    pointer(ippiLBPImageMode3x3_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiLBPImageMode3x3_8u_C1R');

    pointer(ippiFilterWienerGetBufferSize) := GetProcedureAddress(IppiLibHandle,'ippiFilterWienerGetBufferSize');
    pointer(ippiFilterWiener_8u_C1R) := GetProcedureAddress(IppiLibHandle,'ippiFilterWiener_8u_C1R');

    pointer(ippiDeconvLRGetSize_32f) := GetProcedureAddress(IppiLibHandle,'ippiDeconvLRGetSize_32f');
    pointer(ippiDeconvLRInit_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiDeconvLRInit_32f_C1R');
    pointer(ippiDeconvLR_32f_C1R) := GetProcedureAddress(IppiLibHandle,'ippiDeconvLR_32f_C1R');

    // Computer Vision (ippcv.h)
    pointer(ippiSegmentWatershedGetBufferSize_8u16u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiSegmentWatershedGetBufferSize_8u16u_C1R');
    pointer(ippiSegmentWatershed_8u16u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiSegmentWatershed_8u16u_C1IR');
    pointer(ippiBoundSegments_16u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiBoundSegments_16u_C1IR');

    pointer(ippiLabelMarkersGetBufferSize_8u32s_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiLabelMarkersGetBufferSize_8u32s_C1R');
    pointer(ippiLabelMarkersGetBufferSize_16u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiLabelMarkersGetBufferSize_16u_C1R');
    pointer(ippiLabelMarkers_8u32s_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiLabelMarkers_8u32s_C1R');
    pointer(ippiLabelMarkers_16u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiLabelMarkers_16u_C1IR');

    pointer(ippiFilterGaussianGetBufferSize) := GetProcedureAddress(IppcvLibHandle,'ippiFilterGaussianGetBufferSize');
    pointer(ippiFilterGaussianInit) := GetProcedureAddress(IppcvLibHandle,'ippiFilterGaussianInit');
    pointer(ippiFilterGaussianBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiFilterGaussianBorder_8u_C1R');

    pointer(ippiDistanceTransform_3x3_8u16u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiDistanceTransform_3x3_8u16u_C1R');
    pointer(ippiDistanceTransform_3x3_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiDistanceTransform_3x3_8u32f_C1R');
    pointer(ippiTrueDistanceTransformGetBufferSize_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiTrueDistanceTransformGetBufferSize_8u32f_C1R');
    pointer(ippiTrueDistanceTransform_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiTrueDistanceTransform_8u32f_C1R');

    pointer(ippiMorphologyBorderGetSize_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphologyBorderGetSize_8u_C1R');
    pointer(ippiMorphologyBorderInit_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphologyBorderInit_8u_C1R');
    pointer(ippiDilateBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiDilateBorder_8u_C1R');
    pointer(ippiErodeBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiErodeBorder_8u_C1R');
    pointer(ippiMorphAdvGetSize_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphAdvGetSize_8u_C1R');
    pointer(ippiMorphAdvInit_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphAdvInit_8u_C1R');
    pointer(ippiMorphOpenBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphOpenBorder_8u_C1R');
    pointer(ippiMorphCloseBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphCloseBorder_8u_C1R');
    pointer(ippiMorphTophatBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphTophatBorder_8u_C1R');
    pointer(ippiMorphBlackhatBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphBlackhatBorder_8u_C1R');
    pointer(ippiMorphGradientBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiMorphGradientBorder_8u_C1R');

    pointer(ippiHoughLineGetSize_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiHoughLineGetSize_8u_C1R');
    pointer(ippiHoughLine_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiHoughLine_8u32f_C1R');
    pointer(ippiHoughLine_Region_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiHoughLine_Region_8u32f_C1R');

    pointer(ippiCannyBorderGetSize) := GetProcedureAddress(IppcvLibHandle,'ippiCannyBorderGetSize');
    pointer(ippiCannyBorder_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiCannyBorder_8u_C1R');
    pointer(ippiHarrisCornerGetBufferSize) := GetProcedureAddress(IppcvLibHandle,'ippiHarrisCornerGetBufferSize');
    pointer(ippiHarrisCorner_8u32f_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiHarrisCorner_8u32f_C1R');

    pointer(ippiFastNGetSize) := GetProcedureAddress(IppcvLibHandle,'ippiFastNGetSize');
    pointer(ippiFastNInit) := GetProcedureAddress(IppcvLibHandle,'ippiFastNInit');
    pointer(ippiFastNGetBufferSize) := GetProcedureAddress(IppcvLibHandle,'ippiFastNGetBufferSize');
    pointer(ippiFastN_8u_C1R) := GetProcedureAddress(IppcvLibHandle,'ippiFastN_8u_C1R');

    pointer(ippiFloodFillGetSize) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFillGetSize');
    pointer(ippiFloodFillGetSize_Grad) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFillGetSize_Grad');
    pointer(ippiFloodFill_4Con_8u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFill_4Con_8u_C1IR');
    pointer(ippiFloodFill_8Con_8u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFill_8Con_8u_C1IR');
    pointer(ippiFloodFill_Range4Con_8u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFill_Range4Con_8u_C1IR');
    pointer(ippiFloodFill_Range8Con_8u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFill_Range8Con_8u_C1IR');
    pointer(ippiFloodFill_Range8Con_16u_C1IR) := GetProcedureAddress(IppcvLibHandle,'ippiFloodFill_Range8Con_16u_C1IR');
  end;
end;

procedure UnloadIPP;
  function UnloadIppLib(var AHandle: TLibHandle): boolean;
  begin
    Result := System.UnloadLibrary(AHandle);
  end;
begin
  if RefCount > 0 then begin
    Dec(RefCount);
    if RefCount = 0 then begin
      UnloadIppLib(IppcvLibHandle);
      UnloadIppLib(IppiLibHandle);
      UnloadIppLib(IppsLibHandle);
      UnloadIppLib(IppCoreLibHandle);
    end;
  end;
end;
{$ELSE}
procedure LoadIPP(APath: string);
begin
end;
procedure UnloadIPP;
begin
end;
{$ENDIF}

function ROI(width, height: integer): IppiSize; inline;
begin
  Result.width := width;
  Result.height := height;
end;

function POI(x, y: integer): IppiPoint; inline;
begin
  Result.x := x;
  Result.y := y;
end;

{$IFDEF LOAD_DYNAMICALLY}
finalization
  UnloadIPP;
{$ENDIF}

end.

